Class: RSMP::AlarmState

Inherits:
Object
  • Object
show all
Defined in:
lib/rsmp/component/alarm_state.rb

Overview

The state of an alarm on a component. The alarm state is for a particular alarm code, a component typically have an alarm state for each alarm code that is defined for the component type.

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(component:, code:, **options) ⇒ AlarmState

Returns a new instance of AlarmState.



22
23
24
25
26
27
28
29
30
31
32
33
# File 'lib/rsmp/component/alarm_state.rb', line 22

def initialize(component:, code:, **options)
  @component = component
  @component_id = component.c_id
  @code = code
  @suspended = options[:suspended] == true
  @acknowledged = options[:acknowledged] == true
  @active = options[:active] == true
  @timestamp = options[:timestamp]
  @category = options[:category] || 'D'
  @priority = options[:priority] || 2
  @rvs = options[:rvs] || []
end

Instance Attribute Details

#acknowledgedObject (readonly)

Returns the value of attribute acknowledged.



7
8
9
# File 'lib/rsmp/component/alarm_state.rb', line 7

def acknowledged
  @acknowledged
end

#activeObject (readonly)

Returns the value of attribute active.



7
8
9
# File 'lib/rsmp/component/alarm_state.rb', line 7

def active
  @active
end

#categoryObject (readonly)

Returns the value of attribute category.



7
8
9
# File 'lib/rsmp/component/alarm_state.rb', line 7

def category
  @category
end

#codeObject (readonly)

Returns the value of attribute code.



7
8
9
# File 'lib/rsmp/component/alarm_state.rb', line 7

def code
  @code
end

#component_idObject (readonly)

Returns the value of attribute component_id.



7
8
9
# File 'lib/rsmp/component/alarm_state.rb', line 7

def component_id
  @component_id
end

#priorityObject (readonly)

Returns the value of attribute priority.



7
8
9
# File 'lib/rsmp/component/alarm_state.rb', line 7

def priority
  @priority
end

#rvsObject (readonly)

Returns the value of attribute rvs.



7
8
9
# File 'lib/rsmp/component/alarm_state.rb', line 7

def rvs
  @rvs
end

#suspendedObject (readonly)

Returns the value of attribute suspended.



7
8
9
# File 'lib/rsmp/component/alarm_state.rb', line 7

def suspended
  @suspended
end

#timestampObject (readonly)

Returns the value of attribute timestamp.



7
8
9
# File 'lib/rsmp/component/alarm_state.rb', line 7

def timestamp
  @timestamp
end

Class Method Details

.create_from_message(component, message) ⇒ Object



9
10
11
12
13
14
15
16
17
18
19
20
# File 'lib/rsmp/component/alarm_state.rb', line 9

def self.create_from_message(component, message)
  options = {
    timestamp: RSMP::Clock.parse(message.attribute('aTs')),
    acknowledged: message.attribute('ack') == 'Acknowledged',
    suspended: message.attribute('aS') == 'Suspended',
    active: message.attribute('sS') == 'Active',
    category: message.attribute('cat'),
    priority: message.attribute('pri').to_i,
    rvs: message.attribute('rvs')
  }
  new(component: component, code: message.attribute('aCId'), **options)
end

Instance Method Details

#acknowledgeObject



49
50
51
52
53
54
# File 'lib/rsmp/component/alarm_state.rb', line 49

def acknowledge
  change = !@acknowledged
  @acknowledged = true
  update_timestamp if change
  change
end

#activateObject

according to the rsmp core spec, the only time an alarm changes to unanknowledged, is when it’s activated. See: rsmp-nordic.org/rsmp_specifications/core/3.2.0/applicability/basic_structure.html#alarm-status



73
74
75
76
77
78
79
# File 'lib/rsmp/component/alarm_state.rb', line 73

def activate
  change = !@active
  @active = true
  @acknowledged = false
  update_timestamp if change
  change
end

#clear_timestampObject



104
105
106
# File 'lib/rsmp/component/alarm_state.rb', line 104

def clear_timestamp
  @timestamp = nil
end

#deactivateObject



81
82
83
84
85
86
# File 'lib/rsmp/component/alarm_state.rb', line 81

def deactivate
  change = @active
  @active = false
  update_timestamp if change
  change
end

#differ_from_message?(message) ⇒ Boolean

Returns:

  • (Boolean)


92
93
94
95
96
97
98
99
100
101
102
# File 'lib/rsmp/component/alarm_state.rb', line 92

def differ_from_message?(message)
  return true if timestamp_differs?(message)
  return true if acknowledgment_differs?(message)
  return true if suspension_differs?(message)
  return true if activity_differs?(message)
  return true if category_differs?(message)
  return true if priority_differs?(message)

  # return true @rvs = message.attribute('rvs')
  false
end

#older_message?(message) ⇒ Boolean

Returns:

  • (Boolean)


108
109
110
111
112
# File 'lib/rsmp/component/alarm_state.rb', line 108

def older_message?(message)
  return false if @timestamp.nil?

  RSMP::Clock.parse(message.attribute('aTs')) < @timestamp
end

#resumeObject



63
64
65
66
67
68
# File 'lib/rsmp/component/alarm_state.rb', line 63

def resume
  change = @suspended
  @suspended = false
  update_timestamp if change
  change
end

#suspendObject



56
57
58
59
60
61
# File 'lib/rsmp/component/alarm_state.rb', line 56

def suspend
  change = !@suspended
  @suspended = true
  update_timestamp if change
  change
end

#to_hashObject



35
36
37
38
39
40
41
42
43
44
45
46
47
# File 'lib/rsmp/component/alarm_state.rb', line 35

def to_hash
  {
    'cId' => @component_id,
    'aCId' => @code,
    'aTs' => Clock.to_s(@timestamp),
    'ack' => (@acknowledged ? 'Acknowledged' : 'notAcknowledged'),
    'sS' => (@suspended ? 'Suspended' : 'notSuspended'),
    'aS' => (@active ? 'Active' : 'inActive'),
    'cat' => @category,
    'pri' => @priority.to_s,
    'rvs' => @rvs
  }
end

#update_from_message(message) ⇒ Object

update from rsmp message component id, alarm code and specialization are not updated



116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
# File 'lib/rsmp/component/alarm_state.rb', line 116

def update_from_message(message)
  unless differ_from_message? message
    raise RepeatedAlarmError,
          "no changes from previous alarm #{message.m_id_short}"
  end
  raise TimestampError, "timestamp is earlier than previous alarm #{message.m_id_short}" if older_message? message
ensure
  @timestamp = RSMP::Clock.parse message.attribute('aTs')
  @acknowledged = message.attribute('ack') == 'True'
  @suspended = message.attribute('sS') == 'True'
  @active = message.attribute('aS') == 'True'
  @category = message.attribute('cat')
  @priority = message.attribute('pri').to_i
  @rvs = message.attribute('rvs')
end

#update_timestampObject



88
89
90
# File 'lib/rsmp/component/alarm_state.rb', line 88

def update_timestamp
  @timestamp = @component.now
end