Class: RSMP::Message
Overview
Base RSMP message class used to represent parsed and built messages.
Direct Known Subclasses
AggregatedStatus, AggregatedStatusRequest, Alarm, CommandRequest, CommandResponse, ComponentList, Malformed, MessageAcking, StatusRequest, StatusResponse, StatusSubscribe, StatusUnsubscribe, StatusUpdate, Unknown, Version, Watchdog
Constant Summary
RSMP::MessageSxlCodec::SXL_ITEM_SHAPES
Instance Attribute Summary collapse
Class Method Summary
collapse
Instance Method Summary
collapse
#decode_for, #encode_for, included
Constructor Details
#initialize(attributes = {}) ⇒ Message
Returns a new instance of Message.
152
153
154
155
156
157
158
159
|
# File 'lib/rsmp/message.rb', line 152
def initialize(attributes = {})
@timestamp = Time.now
@attributes = { 'mType' => 'rSMsg' }.merge attributes
ensure_message_id
end
|
Instance Attribute Details
#attributes ⇒ Object
this is an internal timestamp recording when we receive/send
7
8
9
|
# File 'lib/rsmp/message.rb', line 7
def attributes
@attributes
end
|
#direction ⇒ Object
Returns the value of attribute direction.
8
9
10
|
# File 'lib/rsmp/message.rb', line 8
def direction
@direction
end
|
#json ⇒ Object
Returns the value of attribute json.
8
9
10
|
# File 'lib/rsmp/message.rb', line 8
def json
@json
end
|
#now ⇒ Object
this is an internal timestamp recording when we receive/send
7
8
9
|
# File 'lib/rsmp/message.rb', line 7
def now
@now
end
|
#out ⇒ Object
this is an internal timestamp recording when we receive/send
7
8
9
|
# File 'lib/rsmp/message.rb', line 7
def out
@out
end
|
#timestamp ⇒ Object
this is an internal timestamp recording when we receive/send
7
8
9
|
# File 'lib/rsmp/message.rb', line 7
def timestamp
@timestamp
end
|
Class Method Details
.bin_to_chars(str) ⇒ Object
117
118
119
120
121
122
123
124
125
126
127
|
# File 'lib/rsmp/message.rb', line 117
def self.bin_to_chars(str)
out = str.gsub(/[^[:print:]]/i, '.')
max = 120
if out.size <= max
out
else
mid = ' ... '
length = ((max - mid.size) / 2) - 1
"#{out[0..length]} ... #{out[(-length - 1)..]}"
end
end
|
.build(attributes, json) ⇒ Object
22
23
24
25
26
27
28
|
# File 'lib/rsmp/message.rb', line 22
def self.build(attributes, json)
validate_message_type attributes
message = create_message_instance(attributes)
message.json = json
message.direction = :in
message
end
|
.build_alarm(attributes) ⇒ Object
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
|
# File 'lib/rsmp/message.rb', line 57
def self.build_alarm(attributes)
case attributes['aSp']
when /^Issue$/i
AlarmIssue.new attributes
when /^Request$/i
AlarmRequest.new attributes
when /^Acknowledge$/i
if attributes['ack'] =~ /^acknowledged$/i
AlarmAcknowledged.new attributes
else
AlarmAcknowledge.new attributes
end
when /^Suspend$/i
if attributes['sS'] =~ /^suspended$/i
AlarmSuspended.new attributes
elsif attributes['sS'] =~ /^notSuspended$/i
AlarmResumed.new attributes
else
AlarmSuspend.new attributes
end
when /^Resume$/i
AlarmResume.new attributes
else
Alarm.new attributes
end
end
|
.create_message_instance(attributes) ⇒ Object
49
50
51
52
53
54
55
|
# File 'lib/rsmp/message.rb', line 49
def self.create_message_instance(attributes)
type = attributes['type']
return build_alarm(attributes) if type == 'Alarm'
klass = message_types[type] || Unknown
klass.new(attributes)
end
|
.make_m_id ⇒ Object
10
11
12
|
# File 'lib/rsmp/message.rb', line 10
def self.make_m_id
SecureRandom.uuid
end
|
.message_types ⇒ Object
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
|
# File 'lib/rsmp/message.rb', line 30
def self.message_types
{
'MessageAck' => MessageAck,
'MessageNotAck' => MessageNotAck,
'Version' => Version,
'ComponentList' => ComponentList,
'AggregatedStatus' => AggregatedStatus,
'AggregatedStatusRequest' => AggregatedStatusRequest,
'Watchdog' => Watchdog,
'CommandRequest' => CommandRequest,
'CommandResponse' => CommandResponse,
'StatusRequest' => StatusRequest,
'StatusResponse' => StatusResponse,
'StatusSubscribe' => StatusSubscribe,
'StatusUnsubscribe' => StatusUnsubscribe,
'StatusUpdate' => StatusUpdate
}
end
|
.parse_attributes(json) ⇒ Object
14
15
16
17
18
19
20
|
# File 'lib/rsmp/message.rb', line 14
def self.parse_attributes(json)
raise ArgumentError unless json
JSON.parse json
rescue JSON::ParserError
raise InvalidPacket, bin_to_chars(json)
end
|
.shorten_m_id(m_id, length = 4) ⇒ Object
96
97
98
99
100
|
# File 'lib/rsmp/message.rb', line 96
def self.shorten_m_id(m_id, length = 4)
return nil unless m_id
m_id[0..(length - 1)]
end
|
.validate_attributes_structure(attributes) ⇒ Object
135
136
137
|
# File 'lib/rsmp/message.rb', line 135
def self.validate_attributes_structure(attributes)
raise MalformedMessage, "JSON must be a Hash, got #{attributes.class} " unless attributes.is_a?(Hash)
end
|
.validate_message_type(attributes) ⇒ Object
129
130
131
132
133
|
# File 'lib/rsmp/message.rb', line 129
def self.validate_message_type(attributes)
validate_attributes_structure(attributes)
validate_mtype_field(attributes)
validate_type_field(attributes)
end
|
.validate_mtype_field(attributes) ⇒ Object
139
140
141
142
143
144
|
# File 'lib/rsmp/message.rb', line 139
def self.validate_mtype_field(attributes)
mtype = attributes['mType']
raise MalformedMessage, "'mType' is missing" unless mtype
raise MalformedMessage, "'mType' must be a String, got #{mtype.class}" unless mtype.is_a?(String)
raise MalformedMessage, "'mType' must be 'rSMsg', got '#{mtype}'" unless mtype == 'rSMsg'
end
|
.validate_type_field(attributes) ⇒ Object
146
147
148
149
150
|
# File 'lib/rsmp/message.rb', line 146
def self.validate_type_field(attributes)
type = attributes['type']
raise MalformedMessage, "'type' is missing" unless type
raise MalformedMessage, "'type' must be a String, got #{type.class}" unless type.is_a?(String)
end
|
Instance Method Details
#attribute(key) ⇒ Object
106
107
108
109
110
111
112
113
114
115
|
# File 'lib/rsmp/message.rb', line 106
def attribute(key)
unless @attributes.key? key
maybe = @attributes.find { |k, _v| k.downcase == key.downcase }
raise MissingAttribute, "attribute '#{maybe.first}' should be named '#{key}'" if maybe
raise MissingAttribute, "missing attribute '#{key}'"
end
@attributes[key]
end
|
#ensure_message_id ⇒ Object
161
162
163
164
|
# File 'lib/rsmp/message.rb', line 161
def ensure_message_id
@attributes['mId'] ||= Message.make_m_id
end
|
#generate_json ⇒ Object
191
192
193
194
195
196
197
198
199
200
|
# File 'lib/rsmp/message.rb', line 191
def generate_json
options = {
array_nl: nil,
object_nl: nil,
space_before: nil,
space: nil
}
@json = JSON.generate @attributes, options
end
|
#inspect ⇒ Object
84
85
86
|
# File 'lib/rsmp/message.rb', line 84
def inspect
"#<#{self.class.name}:#{object_id} m_id: #{m_id_short}>"
end
|
#m_id ⇒ Object
92
93
94
|
# File 'lib/rsmp/message.rb', line 92
def m_id
@attributes['mId']
end
|
#m_id_short ⇒ Object
102
103
104
|
# File 'lib/rsmp/message.rb', line 102
def m_id_short
Message.shorten_m_id @attributes['mId']
end
|
#type ⇒ Object
88
89
90
|
# File 'lib/rsmp/message.rb', line 88
def type
@attributes['type']
end
|
#valid? ⇒ Boolean
187
188
189
|
# File 'lib/rsmp/message.rb', line 187
def valid?
true
end
|
#validate(schemas) ⇒ Object
166
167
168
|
# File 'lib/rsmp/message.rb', line 166
def validate(schemas)
RSMP::Schema.validate attributes, schemas
end
|
#validate!(schemas) ⇒ Object
170
171
172
173
174
175
176
177
|
# File 'lib/rsmp/message.rb', line 170
def validate!(schemas)
validation = validate(schemas)
return self if validation.valid?
err = SchemaError.new validation.message
err.schemas = schemas
raise err
end
|
#validate_id? ⇒ Boolean
183
184
185
|
# File 'lib/rsmp/message.rb', line 183
def validate_id?
!(@attributes['mId'] =~ /[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89ab][a-f0-9]{3}-[a-f0-9]{12}/i).nil?
end
|
#validate_type? ⇒ Boolean
179
180
181
|
# File 'lib/rsmp/message.rb', line 179
def validate_type?
@attributes['mType'] == 'rSMsg'
end
|