Class: RSMP::SupervisorProxy

Inherits:
Proxy
  • Object
show all
Defined in:
lib/rsmp/supervisor_proxy.rb

Constant Summary

Constants inherited from Proxy

Proxy::WRAPPING_DELIMITER

Instance Attribute Summary collapse

Attributes inherited from Proxy

#archive, #collector, #connection_info, #ip, #port, #state, #sxl

Attributes included from Task

#task

Attributes included from Notifier

#listeners

Attributes included from Logging

#archive, #logger

Instance Method Summary collapse

Methods inherited from Proxy

#acknowledge, #acknowledged_first_outgoing, #author, #buffer_message, #check_ack_timeout, #check_ingoing_acknowledged, #check_outgoing_acknowledged, #check_rsmp_version, #check_watchdog_timeout, #clear, #clock, #close, #close_socket, #close_stream, #connected?, #disconnect, #disconnected?, #dont_acknowledge, #dont_expect_acknowledgement, #expect_acknowledgement, #expect_version_message, #extraneous_version, #find_original_for_message, #get_schemas, #inspect, #log, #log_acknowledgement_for_original, #log_acknowledgement_for_unknown, #log_send, #notify_error, #process_ack, #process_deferred, #process_not_ack, #process_packet, #process_watchdog, #read_line, #ready?, #revive, #rsmp_versions, #run_reader, #run_timer, #send_and_optionally_collect, #send_message, #send_version, #send_watchdog, #set_state, #setup, #should_validate_ingoing_message?, #start_reader, #start_timer, #start_watchdog, #state_changed, #stop_reader, #stop_subtasks, #stop_timer, #verify_sequence, #version_acknowledged, #wait_for_reader, #wait_for_state, #watchdog_send_timer, #will_not_handle

Methods included from Task

#initialize_task, #restart, #start, #status, #stop, #stop_subtasks, #wait, #wait_for_condition

Methods included from Inspect

#inspect, #inspector

Methods included from Notifier

#add_listener, #clear_deferred_notify, #deferred_notify, #dequeue_notify, #distribute_error, #initialize_distributor, #inspect, #notify, #notify_without_defer, #remove_listener

Methods included from Logging

#author, #initialize_logging, #log

Constructor Details

#initialize(options) ⇒ SupervisorProxy

Returns a new instance of SupervisorProxy.



10
11
12
13
14
15
16
17
18
19
# File 'lib/rsmp/supervisor_proxy.rb', line 10

def initialize options
  super options
  @site = options[:site]
  @site_settings = @site.site_settings.clone
  @ip = options[:ip]
  @port = options[:port]
  @status_subscriptions = {}
  @sxl = @site_settings['sxl']
  @synthetic_id = Supervisor.build_id_from_ip_port @ip, @port
end

Instance Attribute Details

#siteObject (readonly)

Returns the value of attribute site.



8
9
10
# File 'lib/rsmp/supervisor_proxy.rb', line 8

def site
  @site
end

#supervisor_idObject (readonly)

Returns the value of attribute supervisor_id.



8
9
10
# File 'lib/rsmp/supervisor_proxy.rb', line 8

def supervisor_id
  @supervisor_id
end

Instance Method Details

#acknowledged_first_ingoing(message) ⇒ Object



119
120
121
122
123
124
125
126
127
128
129
# File 'lib/rsmp/supervisor_proxy.rb', line 119

def acknowledged_first_ingoing message
  # TODO
  # aggregateds status should only be send for later version of rsmp
  # to handle verison differences, we probably need inherited classes
  case message.type
    when "Watchdog"
      if @site_settings['send_after_connect']
        send_all_aggregated_status
      end
  end
end

#check_sxl_version(message) ⇒ Object



396
397
# File 'lib/rsmp/supervisor_proxy.rb', line 396

def check_sxl_version message
end

#connectObject

connect to the supervisor and initiate handshake supervisor



54
55
56
57
58
59
60
61
62
63
64
# File 'lib/rsmp/supervisor_proxy.rb', line 54

def connect
  log "Connecting to supervisor at #{@ip}:#{@port}", level: :info
  set_state :connecting
  connect_tcp
  @logger.unmute @ip, @port
  log "Connected to supervisor at #{@ip}:#{@port}", level: :info
rescue SystemCallError => e
  raise ConnectionError.new "Could not connect to supervisor at #{@ip}:#{@port}: Errno #{e.errno} #{e}"
rescue StandardError => e
  raise ConnectionError.new "Error while connecting to supervisor at #{@ip}:#{@port}: #{e}"
end

#connect_tcpObject



71
72
73
74
75
76
77
78
79
80
81
82
83
# File 'lib/rsmp/supervisor_proxy.rb', line 71

def connect_tcp
  @endpoint = Async::IO::Endpoint.tcp(@ip, @port)

  # Async::IO::Endpoint#connect renames the current task. run in a subtask to avoid this
  @task.async do |task|
    task.annotate 'socket task'
    @socket = @endpoint.connect
  end.wait

  @stream = Async::IO::Stream.new(@socket)
  @protocol = Async::IO::Protocol::Line.new(@stream,WRAPPING_DELIMITER) # rsmp messages are json terminated with a form-feed
  set_state :connected
end

#fetch_last_sent_status(component, code, name) ⇒ Object



302
303
304
# File 'lib/rsmp/supervisor_proxy.rb', line 302

def fetch_last_sent_status component, code, name
  @last_status_sent.dig component, code, name if @last_status_sent
end

#handshake_completeObject



85
86
87
88
89
90
# File 'lib/rsmp/supervisor_proxy.rb', line 85

def handshake_complete
  super
  sanitized_sxl_version = RSMP::Schemer.sanitize_version(sxl_version)
  log "Connection to supervisor established, using core #{@rsmp_version}, #{sxl} #{sanitized_sxl_version}", level: :info
  start_watchdog
end

#nodeObject



21
22
23
# File 'lib/rsmp/supervisor_proxy.rb', line 21

def node
  site
end

#process_aggregated_status(message) ⇒ Object



172
173
174
175
176
177
178
# File 'lib/rsmp/supervisor_proxy.rb', line 172

def process_aggregated_status message
  se = message.attribute("se")
  validate_aggregated_status(message,se) == false
  on = set_aggregated_status se
  log "Received #{message.type} status [#{on.join(', ')}]", message: message, level: :log
  acknowledge message
end

#process_aggregated_status_request(message) ⇒ Object



201
202
203
204
205
206
207
# File 'lib/rsmp/supervisor_proxy.rb', line 201

def process_aggregated_status_request message
  log "Received #{message.type}", message: message, level: :log
  component_id = message.attributes["cId"]
  component = @site.find_component component_id
  acknowledge message
  send_aggregated_status component
end

#process_alarm(message) ⇒ Object



180
181
182
183
184
185
186
# File 'lib/rsmp/supervisor_proxy.rb', line 180

def process_alarm message
  alarm_code = message.attribute("aCId")
  asp = message.attribute("aSp")
  status = ["ack","aS","sS"].map { |key| message.attribute(key) }.join(',')
  log "Received #{message.type}, #{alarm_code} #{asp} [#{status}]", message: message, level: :log
  acknowledge message
end

#process_command_request(message) ⇒ Object



209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
# File 'lib/rsmp/supervisor_proxy.rb', line 209

def process_command_request message
  log "Received #{message.type}", message: message, level: :log
  component_id = message.attributes["cId"]
  component = @site.find_component component_id
  commands = simplify_command_requests message.attributes["arg"]
  commands.each_pair do |command_code,arg|
    component.handle_command command_code,arg
  end

  rvs = message.attributes["arg"].map do |item|
    item = item.dup.merge('age'=>'recent')
    item.delete 'cO'
    item
  end
  response = CommandResponse.new({
    "cId"=>component_id,
    "cTS"=>clock.to_s,
    "rvs"=>rvs
  })
  acknowledge message
  send_message response
end

#process_message(message) ⇒ Object



92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
# File 'lib/rsmp/supervisor_proxy.rb', line 92

def process_message message
  case message
    when Alarm
    when StatusResponse
    when StatusUpdate
    when AggregatedStatus
      will_not_handle message
    when AggregatedStatusRequest
      process_aggregated_status_request message
    when CommandRequest
      process_command_request message
    when CommandResponse
      process_command_response message
    when StatusRequest
      process_status_request message
    when StatusSubscribe
      process_status_subcribe message
    when StatusUnsubscribe
      process_status_unsubcribe message
    else
      super message
  end
rescue UnknownComponent, UnknownCommand, UnknownStatus,
       MessageRejected, MissingAttribute => e
  dont_acknowledge message, '', e.to_s
end

#process_status_request(message, options = {}) ⇒ Object



232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
# File 'lib/rsmp/supervisor_proxy.rb', line 232

def process_status_request message, options={}
  component_id = message.attributes["cId"]
  component = @site.find_component component_id
  log "Received #{message.type}", message: message, level: :log
  sS = message.attributes["sS"].map do |arg|
    value, quality =  component.get_status arg['sCI'], arg['n']
    { "s" => value.to_s, "q" => quality.to_s }.merge arg
  end
  response = StatusResponse.new({
    "cId"=>component_id,
    "sTs"=>clock.to_s,
    "sS"=>sS,
    "mId" => options[:m_id]
  })
  acknowledge message
  send_message response
end

#process_status_subcribe(message) ⇒ Object



250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
# File 'lib/rsmp/supervisor_proxy.rb', line 250

def process_status_subcribe message
  log "Received #{message.type}", message: message, level: :log

  # @status_subscriptions is organized by component/code/name, for example:
  #
  # {"AA+BBCCC=DDDEE002"=>{"S001"=>["number"]}}
  #
  # This is done to make it easy to send a single status update
  # for each component, containing all the requested statuses

  update_list = {}
  component = message.attributes["cId"]
  @status_subscriptions[component] ||= {}
  update_list[component] ||= {}
  now = Time.now  # internal timestamp
  subs = @status_subscriptions[component]

  message.attributes["sS"].each do |arg|
    sCI = arg["sCI"]
    subcription = {interval: arg["uRt"].to_i, last_sent_at: now}
    subs[sCI] ||= {}
    subs[sCI][arg["n"]] = subcription
    update_list[component][sCI] ||= []
    update_list[component][sCI] << arg["n"]
  end
  acknowledge message
  send_status_updates update_list   # send status after subscribing is accepted
end

#process_status_unsubcribe(message) ⇒ Object



279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
# File 'lib/rsmp/supervisor_proxy.rb', line 279

def process_status_unsubcribe message
  log "Received #{message.type}", message: message, level: :log
  component = message.attributes["cId"]

  subs = @status_subscriptions[component]
  if subs
    message.attributes["sS"].each do |arg|
      sCI = arg["sCI"]
      if subs[sCI]
        subs[sCI].delete arg["n"]
        subs.delete(sCI) if subs[sCI].empty?
      end
    end
    @status_subscriptions.delete(component) if subs.empty?
  end
  acknowledge message
end

#process_version(message) ⇒ Object



388
389
390
391
392
393
394
# File 'lib/rsmp/supervisor_proxy.rb', line 388

def process_version message
  return extraneous_version message if @version_determined
  check_rsmp_version message
  check_sxl_version message
  @site_id = Supervisor.build_id_from_ip_port @ip, @port
  version_accepted message
end

#reconnect_delayObject



139
140
141
142
143
144
145
146
# File 'lib/rsmp/supervisor_proxy.rb', line 139

def reconnect_delay
  return false if @site_settings['intervals']['reconnect'] == :no
  interval = @site_settings['intervals']['reconnect']
  log "Will try to reconnect again every #{interval} seconds...", level: :info
  @logger.mute @ip, @port
  @task.sleep interval
  true
end

#runObject

handle communication if disconnected, then try to reconnect



27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
# File 'lib/rsmp/supervisor_proxy.rb', line 27

def run
  loop do
    connect
    start_reader
    start_handshake
    wait_for_reader   # run until disconnected
    break if reconnect_delay == false
  rescue Restart
    @logger.mute @ip, @port
    raise
  rescue RSMP::ConnectionError => e
    log e, level: :error
    break if reconnect_delay == false
  rescue StandardError => e
    notify_error e, level: :internal
    break if reconnect_delay == false
  ensure
    close
    stop_subtasks
  end
end

#send_aggregated_status(component, options = {}) ⇒ Object



156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
# File 'lib/rsmp/supervisor_proxy.rb', line 156

def send_aggregated_status component, options={}
  m_id = options[:m_id] || RSMP::Message.make_m_id
  message = AggregatedStatus.new({
    "aSTS" => clock.to_s,
    "cId" =>  component.c_id,
    "fP" => nil,
    "fS" => nil,
    "se" => component.aggregated_status_bools,
    "mId" => m_id,
  })

  send_and_optionally_collect message, options do |collect_options|
    Collector.new self, collect_options.merge(task:@task, type: 'MessageAck')
  end
end

#send_all_aggregated_statusObject



131
132
133
134
135
136
137
# File 'lib/rsmp/supervisor_proxy.rb', line 131

def send_all_aggregated_status
  @site.components.each_pair do |c_id,component|
    if component.grouped
      send_aggregated_status component
    end
  end
end

#send_status_updates(update_list) ⇒ Object



356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
# File 'lib/rsmp/supervisor_proxy.rb', line 356

def send_status_updates update_list
  now = clock.to_s
  update_list.each_pair do |component_id,by_code|
    component = @site.find_component component_id
    sS = []
    by_code.each_pair do |code,names|
      names.map do |status_name,value|
        if value
          quality = 'recent'
        else
          value,quality = component.get_status code, status_name
        end
        sS << { "sCI" => code,
                 "n" => status_name,
                 "s" => value.to_s,
                 "q" => quality }
      end
    end
    update = StatusUpdate.new({
      "cId"=>component_id,
      "sTs"=>now,
      "sS"=>sS
    })
    send_message update
    store_last_sent_status update
  end
end

#simplify_command_requests(arg) ⇒ Object

reorganize rmsp command request arg attribute:

“cCI”:“M0002”,“cO”:“setPlan”,“n”:“status”,“v”:“True”,“cCI”:“M0002”,“cO”:“setPlan”,“n”:“securityCode”,“v”:“5678”,“cCI”:“M0002”,“cO”:“setPlan”,“n”:“timeplan”,“v”:“3”

into the simpler, but equivalent: “securityCode”=>“5678”, “timeplan”=>“3”}



192
193
194
195
196
197
198
199
# File 'lib/rsmp/supervisor_proxy.rb', line 192

def simplify_command_requests arg
  sorted = {}
  arg.each do |item|
    sorted[item['cCI']] ||= {}
    sorted[item['cCI']][item['n']] = item['v']
  end
  sorted
end

#start_handshakeObject



49
50
51
# File 'lib/rsmp/supervisor_proxy.rb', line 49

def start_handshake
  send_version @site_settings['site_id'], @site_settings["rsmp_versions"]
end

#status_update_timer(now) ⇒ Object



317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
# File 'lib/rsmp/supervisor_proxy.rb', line 317

def status_update_timer now
  update_list = {}
  # go through subscriptons and build a similarly organized list,
  # that only contains what should be send

  @status_subscriptions.each_pair do |component,by_code|
    component_object = @site.find_component component
    by_code.each_pair do |code,by_name|
      by_name.each_pair do |name,subscription|
        current = nil
        should_send = false
        if subscription[:interval] == 0
          # send as soon as the data changes
          if component_object
            current, age = *(component_object.get_status code, name)
            current = current.to_s
          end
          last_sent = fetch_last_sent_status component, code, name
          if current != last_sent
            should_send = true
          end
        else
          # send at regular intervals
          if subscription[:last_sent_at] == nil || (now - subscription[:last_sent_at]) >= subscription[:interval]
            should_send = true
          end
        end
        if should_send
          subscription[:last_sent_at] = now
          update_list[component] ||= {}
          update_list[component][code] ||= {}
          update_list[component][code][name] = current
       end
      end
    end
  end
  send_status_updates update_list
end

#stop_taskObject



66
67
68
69
# File 'lib/rsmp/supervisor_proxy.rb', line 66

def stop_task
  super
  @last_status_sent = nil
end

#store_last_sent_status(message) ⇒ Object



306
307
308
309
310
311
312
313
314
315
# File 'lib/rsmp/supervisor_proxy.rb', line 306

def store_last_sent_status message
  component_id = message.attribute('cId')
  @last_status_sent ||= {}
  @last_status_sent[component_id] ||= {}
  message.attribute('sS').each do |item|
    sCI, n, s = item['sCI'], item['n'], item['s']
    @last_status_sent[component_id][sCI] ||= {}
    @last_status_sent[component_id][sCI][n] = s
  end
end

#sxl_versionObject



384
385
386
# File 'lib/rsmp/supervisor_proxy.rb', line 384

def sxl_version
  @site_settings['sxl_version']
end

#timer(now) ⇒ Object



297
298
299
300
# File 'lib/rsmp/supervisor_proxy.rb', line 297

def timer now
  super
  status_update_timer now if ready?
end

#version_accepted(message) ⇒ Object



148
149
150
151
152
153
154
# File 'lib/rsmp/supervisor_proxy.rb', line 148

def version_accepted message
  log "Received Version message, using RSMP #{@rsmp_version}", message: message, level: :log
  start_timer
  acknowledge message
  handshake_complete
  @version_determined = true
end