Class: RSMP::SupervisorProxy

Inherits:
Proxy
  • Object
show all
Defined in:
lib/rsmp/supervisor_proxy.rb

Constant Summary

Constants inherited from Proxy

Proxy::WRAPPING_DELIMITER

Instance Attribute Summary collapse

Attributes inherited from Proxy

#archive, #collector, #connection_info, #ip, #port, #state, #sxl, #task

Attributes included from Notifier

#listeners

Attributes included from Logging

#archive, #logger

Instance Method Summary collapse

Methods inherited from Proxy

#acknowledge, #acknowledged_first_outgoing, #author, #buffer_message, #check_ack_timeout, #check_ingoing_acknowledged, #check_outgoing_acknowledged, #check_rsmp_version, #check_watchdog_timeout, #clear, #clock, #close_socket, #collect, #connected?, #dont_acknowledge, #dont_expect_acknowledgement, #expect_acknowledgement, #expect_version_message, #extraneous_version, #find_original_for_message, #get_schemas, #inspect, #log, #log_acknowledgement_for_original, #log_acknowledgement_for_unknown, #log_send, #notify_error, #process_ack, #process_deferred, #process_not_ack, #process_packet, #process_watchdog, #ready?, #revive, #rsmp_versions, #run, #send_message, #send_version, #send_watchdog, #set_state, #setup, #should_validate_ingoing_message?, #start_reader, #start_timer, #start_watchdog, #stop_tasks, #verify_sequence, #version_acknowledged, #wait_for_acknowledgement, #wait_for_state, #watchdog_send_timer, #will_not_handle

Methods included from Inspect

#inspect, #inspector

Methods included from Notifier

#add_listener, #clear_deferred_notify, #deferred_notify, #dequeue_notify, #distribute_error, #initialize_distributor, #inspect, #notify, #notify_without_defer, #remove_listener

Methods included from Wait

#wait_for

Methods included from Logging

#author, #initialize_logging, #log

Constructor Details

#initialize(options) ⇒ SupervisorProxy

Returns a new instance of SupervisorProxy.



10
11
12
13
14
15
16
17
18
19
# File 'lib/rsmp/supervisor_proxy.rb', line 10

def initialize options
  super options
  @site = options[:site]
  @site_settings = @site.site_settings.clone
  @ip = options[:ip]
  @port = options[:port]
  @status_subscriptions = {}
  @sxl = @site_settings['sxl']
  @synthetic_id = Supervisor.build_id_from_ip_port @ip, @port
end

Instance Attribute Details

#siteObject (readonly)

Returns the value of attribute site.



8
9
10
# File 'lib/rsmp/supervisor_proxy.rb', line 8

def site
  @site
end

#supervisor_idObject (readonly)

Returns the value of attribute supervisor_id.



8
9
10
# File 'lib/rsmp/supervisor_proxy.rb', line 8

def supervisor_id
  @supervisor_id
end

Instance Method Details

#acknowledged_first_ingoing(message) ⇒ Object



96
97
98
99
100
101
102
103
104
105
106
# File 'lib/rsmp/supervisor_proxy.rb', line 96

def acknowledged_first_ingoing message
  # TODO
  # aggregateds status should only be send for later version of rsmp
  # to handle verison differences, we probably need inherited classes
  case message.type
    when "Watchdog"
      if @site_settings['send_after_connect']
        send_all_aggregated_status
      end
  end
end

#check_sxl_version(message) ⇒ Object



378
379
# File 'lib/rsmp/supervisor_proxy.rb', line 378

def check_sxl_version message
end

#connectObject



54
55
56
57
58
59
60
# File 'lib/rsmp/supervisor_proxy.rb', line 54

def connect
  return if @socket
  @endpoint = Async::IO::Endpoint.tcp(@ip, @port)
  @socket = @endpoint.connect
  @stream = Async::IO::Stream.new(@socket)
  @protocol = Async::IO::Protocol::Line.new(@stream,WRAPPING_DELIMITER) # rsmp messages are json terminated with a form-feed
end

#connection_completeObject



62
63
64
65
66
67
# File 'lib/rsmp/supervisor_proxy.rb', line 62

def connection_complete
  super
  sanitized_sxl_version = RSMP::Schemer.sanitize_version(sxl_version)
  log "Connection to supervisor established, using core #{@rsmp_version}, #{sxl} #{sanitized_sxl_version}", level: :info
  start_watchdog
end

#fetch_last_sent_status(component, code, name) ⇒ Object



284
285
286
# File 'lib/rsmp/supervisor_proxy.rb', line 284

def fetch_last_sent_status component, code, name
  @last_status_sent.dig component, code, name if @last_status_sent
end

#nodeObject



21
22
23
# File 'lib/rsmp/supervisor_proxy.rb', line 21

def node
  site
end

#process_aggregated_status(message) ⇒ Object



154
155
156
157
158
159
160
# File 'lib/rsmp/supervisor_proxy.rb', line 154

def process_aggregated_status message
  se = message.attribute("se")
  validate_aggregated_status(message,se) == false
  on = set_aggregated_status se
  log "Received #{message.type} status [#{on.join(', ')}]", message: message, level: :log
  acknowledge message
end

#process_aggregated_status_request(message) ⇒ Object



183
184
185
186
187
188
189
# File 'lib/rsmp/supervisor_proxy.rb', line 183

def process_aggregated_status_request message
  log "Received #{message.type}", message: message, level: :log
  component_id = message.attributes["cId"]
  component = @site.find_component component_id
  acknowledge message
  send_aggregated_status component
end

#process_alarm(message) ⇒ Object



162
163
164
165
166
167
168
# File 'lib/rsmp/supervisor_proxy.rb', line 162

def process_alarm message
  alarm_code = message.attribute("aCId")
  asp = message.attribute("aSp")
  status = ["ack","aS","sS"].map { |key| message.attribute(key) }.join(',')
  log "Received #{message.type}, #{alarm_code} #{asp} [#{status}]", message: message, level: :log
  acknowledge message
end

#process_command_request(message) ⇒ Object



191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
# File 'lib/rsmp/supervisor_proxy.rb', line 191

def process_command_request message
  log "Received #{message.type}", message: message, level: :log
  component_id = message.attributes["cId"]
  component = @site.find_component component_id
  commands = simplify_command_requests message.attributes["arg"]
  commands.each_pair do |command_code,arg|
    component.handle_command command_code,arg
  end

  rvs = message.attributes["arg"].map do |item|
    item = item.dup.merge('age'=>'recent')
    item.delete 'cO'
    item
  end
  response = CommandResponse.new({
    "cId"=>component_id,
    "cTS"=>clock.to_s,
    "rvs"=>rvs
  })
  acknowledge message
  send_message response
end

#process_message(message) ⇒ Object



69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
# File 'lib/rsmp/supervisor_proxy.rb', line 69

def process_message message
  case message
    when Alarm
    when StatusResponse
    when StatusUpdate
    when AggregatedStatus
      will_not_handle message
    when AggregatedStatusRequest
      process_aggregated_status_request message
    when CommandRequest
      process_command_request message
    when CommandResponse
      process_command_response message
    when StatusRequest
      process_status_request message
    when StatusSubscribe
      process_status_subcribe message
    when StatusUnsubscribe
      process_status_unsubcribe message
    else
      super message
  end
rescue UnknownComponent, UnknownCommand, UnknownStatus,
       MessageRejected, MissingAttribute => e
  dont_acknowledge message, '', e.to_s
end

#process_status_request(message, options = {}) ⇒ Object



214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
# File 'lib/rsmp/supervisor_proxy.rb', line 214

def process_status_request message, options={}
  component_id = message.attributes["cId"]
  component = @site.find_component component_id
  log "Received #{message.type}", message: message, level: :log
  sS = message.attributes["sS"].map do |arg|
    value, quality =  component.get_status arg['sCI'], arg['n']
    { "s" => value.to_s, "q" => quality.to_s }.merge arg
  end
  response = StatusResponse.new({
    "cId"=>component_id,
    "sTs"=>clock.to_s,
    "sS"=>sS,
    "mId" => options[:m_id]
  })
  acknowledge message
  send_message response
end

#process_status_subcribe(message) ⇒ Object



232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
# File 'lib/rsmp/supervisor_proxy.rb', line 232

def process_status_subcribe message
  log "Received #{message.type}", message: message, level: :log

  # @status_subscriptions is organized by component/code/name, for example:
  #
  # {"AA+BBCCC=DDDEE002"=>{"S001"=>["number"]}}
  #
  # This is done to make it easy to send a single status update
  # for each component, containing all the requested statuses

  update_list = {}
  component = message.attributes["cId"]
  @status_subscriptions[component] ||= {}
  update_list[component] ||= {} 
  now = Time.now  # internal timestamp
  subs = @status_subscriptions[component]

  message.attributes["sS"].each do |arg|
    sCI = arg["sCI"]
    subcription = {interval: arg["uRt"].to_i, last_sent_at: now}
    subs[sCI] ||= {}
    subs[sCI][arg["n"]] = subcription
    update_list[component][sCI] ||= []
    update_list[component][sCI] << arg["n"]
  end
  acknowledge message
  send_status_updates update_list   # send status after subscribing is accepted
end

#process_status_unsubcribe(message) ⇒ Object



261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
# File 'lib/rsmp/supervisor_proxy.rb', line 261

def process_status_unsubcribe message
  log "Received #{message.type}", message: message, level: :log
  component = message.attributes["cId"]

  subs = @status_subscriptions[component]
  if subs
    message.attributes["sS"].each do |arg|
      sCI = arg["sCI"]
      if subs[sCI]
        subs[sCI].delete arg["n"]
        subs.delete(sCI) if subs[sCI].empty?
      end
    end
    @status_subscriptions.delete(component) if subs.empty?
  end
  acknowledge message
end

#process_version(message) ⇒ Object



370
371
372
373
374
375
376
# File 'lib/rsmp/supervisor_proxy.rb', line 370

def process_version message
  return extraneous_version message if @version_determined
  check_rsmp_version message
  check_sxl_version message
  @site_id = Supervisor.build_id_from_ip_port @ip, @port
  version_accepted message
end

#reconnect_delayObject



116
117
118
119
120
# File 'lib/rsmp/supervisor_proxy.rb', line 116

def reconnect_delay
  interval = @site_settings['intervals']['reconnect']
  log "Waiting #{interval} seconds before trying to reconnect", level: :info
  @task.sleep interval
end

#retry_noticeObject



41
42
43
44
45
46
# File 'lib/rsmp/supervisor_proxy.rb', line 41

def retry_notice
  unless @site.site_settings['intervals']['reconnect'] == :no
    log "Will try to reconnect again every #{@site.site_settings['intervals']['reconnect']} seconds..", level: :info
    @logger.mute @ip, @port
  end
end

#send_aggregated_status(component, options = {}) ⇒ Object



130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
# File 'lib/rsmp/supervisor_proxy.rb', line 130

def send_aggregated_status component, options={}
  m_id = options[:m_id] || RSMP::Message.make_m_id
  message = AggregatedStatus.new({
    "aSTS" => clock.to_s,
    "cId" =>  component.c_id,
    "fP" => nil,
    "fS" => nil,
    "se" => component.aggregated_status_bools,
    "mId" => m_id,
  })

  if options[:collect]
    result = nil
    task = @task.async do |task|
      wait_for_acknowledgement task, options[:collect], m_id
    end
    send_message message, validate: options[:validate]
    return message, task.wait
  else
    send_message message, validate: options[:validate]
    message
  end
end

#send_all_aggregated_statusObject



108
109
110
111
112
113
114
# File 'lib/rsmp/supervisor_proxy.rb', line 108

def send_all_aggregated_status
  @site.components.each_pair do |c_id,component|
    if component.grouped
      send_aggregated_status component
    end
  end
end

#send_status_updates(update_list) ⇒ Object



338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
# File 'lib/rsmp/supervisor_proxy.rb', line 338

def send_status_updates update_list
  now = clock.to_s
  update_list.each_pair do |component_id,by_code|
    component = @site.find_component component_id
    sS = []
    by_code.each_pair do |code,names|
      names.map do |status_name,value|
        if value
          quality = 'recent'
        else
          value,quality = component.get_status code, status_name
        end
        sS << { "sCI" => code,
                 "n" => status_name,
                 "s" => value.to_s,
                 "q" => quality }
      end
    end
    update = StatusUpdate.new({
      "cId"=>component_id,
      "sTs"=>now,
      "sS"=>sS
    })
    send_message update
    store_last_sent_status update
  end
end

#simplify_command_requests(arg) ⇒ Object

reorganize rmsp command request arg attribute:

“cCI”:“M0002”,“cO”:“setPlan”,“n”:“status”,“v”:“True”,“cCI”:“M0002”,“cO”:“setPlan”,“n”:“securityCode”,“v”:“5678”,“cCI”:“M0002”,“cO”:“setPlan”,“n”:“timeplan”,“v”:“3”

into the simpler, but equivalent: “securityCode”=>“5678”, “timeplan”=>“3”}



174
175
176
177
178
179
180
181
# File 'lib/rsmp/supervisor_proxy.rb', line 174

def simplify_command_requests arg
  sorted = {}
  arg.each do |item|
    sorted[item['cCI']] ||= {}
    sorted[item['cCI']][item['n']] = item['v']
  end
  sorted
end

#startObject



25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
# File 'lib/rsmp/supervisor_proxy.rb', line 25

def start
  log "Connecting to supervisor at #{@ip}:#{@port}", level: :info
  super
  connect
  @logger.unmute @ip, @port
  log "Connected to supervisor at #{@ip}:#{@port}", level: :info
  start_reader
  send_version @site_settings['site_id'], @site_settings["rsmp_versions"]
rescue SystemCallError => e
  log "Could not connect to supervisor at #{@ip}:#{@port}: Errno #{e.errno} #{e}", level: :error
  retry_notice
rescue StandardError => e
  log "Error while connecting to supervisor at #{@ip}:#{@port}: #{e}", level: :error
  retry_notice
end

#status_update_timer(now) ⇒ Object



299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
# File 'lib/rsmp/supervisor_proxy.rb', line 299

def status_update_timer now
  update_list = {}
  # go through subscriptons and build a similarly organized list,
  # that only contains what should be send

  @status_subscriptions.each_pair do |component,by_code|
    component_object = @site.find_component component
    by_code.each_pair do |code,by_name|
      by_name.each_pair do |name,subscription|
        current = nil
        should_send = false
        if subscription[:interval] == 0 
          # send as soon as the data changes
          if component_object
            current, age = *(component_object.get_status code, name)
            current = current.to_s
          end
          last_sent = fetch_last_sent_status component, code, name
          if current != last_sent
            should_send = true
          end
        else
          # send at regular intervals
          if subscription[:last_sent_at] == nil || (now - subscription[:last_sent_at]) >= subscription[:interval]
            should_send = true
          end
        end
        if should_send
          subscription[:last_sent_at] = now
          update_list[component] ||= {}
          update_list[component][code] ||= {}
          update_list[component][code][name] = current
       end
      end
    end
  end
  send_status_updates update_list
end

#stopObject



48
49
50
51
52
# File 'lib/rsmp/supervisor_proxy.rb', line 48

def stop
  log "Closing connection to supervisor", level: :info
  super
  @last_status_sent = nil
end

#store_last_sent_status(message) ⇒ Object



288
289
290
291
292
293
294
295
296
297
# File 'lib/rsmp/supervisor_proxy.rb', line 288

def store_last_sent_status message
  component_id = message.attribute('cId')
  @last_status_sent ||= {}
  @last_status_sent[component_id] ||= {}
  message.attribute('sS').each do |item|
    sCI, n, s = item['sCI'], item['n'], item['s']
    @last_status_sent[component_id][sCI] ||= {}
    @last_status_sent[component_id][sCI][n] = s
  end
end

#sxl_versionObject



366
367
368
# File 'lib/rsmp/supervisor_proxy.rb', line 366

def sxl_version
  @site_settings['sxl_version']
end

#timer(now) ⇒ Object



279
280
281
282
# File 'lib/rsmp/supervisor_proxy.rb', line 279

def timer now
  super
  status_update_timer now if ready?
end

#version_accepted(message) ⇒ Object



122
123
124
125
126
127
128
# File 'lib/rsmp/supervisor_proxy.rb', line 122

def version_accepted message
  log "Received Version message, using RSMP #{@rsmp_version}", message: message, level: :log
  start_timer
  acknowledge message
  connection_complete
  @version_determined = true
end