Class: RSMP::TLC::TrafficController

Inherits:
Component show all
Defined in:
lib/rsmp/tlc/traffic_controller.rb

Overview

TrafficController is the main component of a TrafficControllerSite. It handles all command and status for the main component, and keeps track of signal plans, detector logics, inputs, etc. which do not have dedicated components.

Constant Summary

Constants inherited from ComponentBase

ComponentBase::AGGREGATED_STATUS_KEYS

Instance Attribute Summary collapse

Attributes inherited from ComponentBase

#aggregated_status, #aggregated_status_bools, #alarms, #c_id, #grouped, #node, #ntsOId, #statuses, #xNId

Instance Method Summary collapse

Methods inherited from Component

#activate_alarm, #deactivate_alarm, #resume_alarm, #suspend_alarm

Methods inherited from ComponentBase

#aggregated_status_changed, #clear_aggregated_status, #get_alarm_state, #log, #set_aggregated_status, #set_aggregated_status_bools

Methods included from Inspect

#inspect, #inspector

Constructor Details

#initialize(node:, id:, ntsOId: nil, xNId: nil, cycle_time: 10, signal_plans:, startup_sequence:, live_output: nil, inputs: {}) ⇒ TrafficController

Returns a new instance of TrafficController.



12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
# File 'lib/rsmp/tlc/traffic_controller.rb', line 12

def initialize node:, id:, ntsOId: nil, xNId: nil, cycle_time: 10, signal_plans:,
    startup_sequence:, live_output:nil, inputs:{}
  super node: node, id: id, ntsOId: ntsOId, xNId: xNId, grouped: true
  @signal_groups = []
  @detector_logics = []
  @plans = signal_plans
  @cycle_time = cycle_time
  @num_traffic_situations = 1

  if inputs
    num_inputs = inputs['total']
    @input_programming = inputs['programming']
  else
    @input_programming = nil
  end
  @inputs = TLC::Inputs.new num_inputs || 8

  @startup_sequence = startup_sequence
  @live_output = live_output
  reset
end

Instance Attribute Details

#cycle_counterObject (readonly)

Returns the value of attribute cycle_counter.



8
9
10
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8

def cycle_counter
  @cycle_counter
end

#cycle_timeObject (readonly)

Returns the value of attribute cycle_time.



8
9
10
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8

def cycle_time
  @cycle_time
end

#functional_positionObject (readonly)

Returns the value of attribute functional_position.



8
9
10
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8

def functional_position
  @functional_position
end

#planObject (readonly)

Returns the value of attribute plan.



8
9
10
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8

def plan
  @plan
end

#posObject (readonly)

Returns the value of attribute pos.



8
9
10
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8

def pos
  @pos
end

#startup_sequenceObject (readonly)

Returns the value of attribute startup_sequence.



8
9
10
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8

def startup_sequence
  @startup_sequence
end

#startup_sequence_activeObject (readonly)

Returns the value of attribute startup_sequence_active.



8
9
10
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8

def startup_sequence_active
  @startup_sequence_active
end

#startup_sequence_posObject (readonly)

Returns the value of attribute startup_sequence_pos.



8
9
10
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8

def startup_sequence_pos
  @startup_sequence_pos
end

Instance Method Details

#add_detector_logic(logic) ⇒ Object



104
105
106
# File 'lib/rsmp/tlc/traffic_controller.rb', line 104

def add_detector_logic logic
  @detector_logics << logic
end

#add_signal_group(group) ⇒ Object



100
101
102
# File 'lib/rsmp/tlc/traffic_controller.rb', line 100

def add_signal_group group
  @signal_groups << group
end

#bool_string_to_digit(bool) ⇒ Object



401
402
403
404
405
406
407
408
409
410
# File 'lib/rsmp/tlc/traffic_controller.rb', line 401

def bool_string_to_digit bool
  case bool
    when 'True'
      '1'
    when 'False'
      '0'
    else
      raise RSMP::MessageRejected.new "Invalid boolean '#{bool}', must be 'True' or 'False'"
  end
end

#bool_to_digit(bool) ⇒ Object



412
413
414
# File 'lib/rsmp/tlc/traffic_controller.rb', line 412

def bool_to_digit bool
  bool ?  '1' : '0'
end

#check_functional_position_timeoutObject



149
150
151
152
153
154
155
156
# File 'lib/rsmp/tlc/traffic_controller.rb', line 149

def check_functional_position_timeout
  return unless @functional_position_timeout
  if clock.now >= @functional_position_timeout
    switch_functional_position @previous_functional_position, reverting: true, source: 'calendar_clock'
    @functional_position_timeout = nil
    @previous_functional_position = nil
  end
end

#clockObject



87
88
89
# File 'lib/rsmp/tlc/traffic_controller.rb', line 87

def clock
  node.clock
end

#current_planObject



91
92
93
94
95
96
97
98
# File 'lib/rsmp/tlc/traffic_controller.rb', line 91

def current_plan
  # TODO plan 0 should means use time table
  if @plans
    @plans[ plan ] || @plans.values.first
  else
    nil
  end
end

#dark?Boolean

Returns:

  • (Boolean)


75
76
77
# File 'lib/rsmp/tlc/traffic_controller.rb', line 75

def dark?
  @function_position == 'Dark'
end

#end_startup_sequenceObject



172
173
174
175
176
# File 'lib/rsmp/tlc/traffic_controller.rb', line 172

def end_startup_sequence
  @startup_sequence_active = false
  @startup_sequence_initiated_at = nil
  @startup_sequence_pos = nil
end

#find_plan(plan_nr) ⇒ Object



344
345
346
347
348
# File 'lib/rsmp/tlc/traffic_controller.rb', line 344

def find_plan plan_nr
  plan = @plans[plan_nr.to_i]
  raise InvalidMessage.new "unknown signal plan #{plan_nr}, known only [#{@plans.keys.join(',')}]" unless plan
  plan
end

#format_signal_group_statusObject



235
236
237
# File 'lib/rsmp/tlc/traffic_controller.rb', line 235

def format_signal_group_status
  @signal_groups.map { |group| group.state }.join
end

#get_priority_listObject



134
135
136
137
138
139
140
141
142
# File 'lib/rsmp/tlc/traffic_controller.rb', line 134

def get_priority_list
  @signal_priorities.map do |priority|
    {
      "r" => priority.id,
      "t" => RSMP::Clock.to_s(priority.updated),
      "s" => priority.state
    }
  end
end

#get_status(code, name = nil, options = {}) ⇒ Object



556
557
558
559
560
561
562
563
564
565
566
567
568
569
# File 'lib/rsmp/tlc/traffic_controller.rb', line 556

def get_status code, name=nil, options={}
  case code
  when 'S0001', 'S0002', 'S0003', 'S0004', 'S0005', 'S0006', 'S0007',
       'S0008', 'S0009', 'S0010', 'S0011', 'S0012', 'S0013', 'S0014',
       'S0015', 'S0016', 'S0017', 'S0018', 'S0019', 'S0020', 'S0021',
       'S0022', 'S0023', 'S0024', 'S0026', 'S0027', 'S0028',
       'S0029', 'S0030', 'S0031', 'S0032', 'S0033',
       'S0091', 'S0092', 'S0095', 'S0096', 'S0097',
       'S0205', 'S0206', 'S0207', 'S0208'
    return send("handle_#{code.downcase}", code, name, options)
  else
    raise InvalidMessage.new "unknown status code #{code}"
  end
end

#handle_command(command_code, arg, options = {}) ⇒ Object



239
240
241
242
243
244
245
246
247
248
249
250
# File 'lib/rsmp/tlc/traffic_controller.rb', line 239

def handle_command command_code, arg, options={}
  case command_code
  when 'M0001', 'M0002', 'M0003', 'M0004', 'M0005', 'M0006', 'M0007',
       'M0012', 'M0013', 'M0014', 'M0015', 'M0016', 'M0017', 'M0018',
       'M0019', 'M0020', 'M0021', 'M0022', 'M0023',
       'M0103', 'M0104'

    return send("handle_#{command_code.downcase}", arg, options)
  else
    raise UnknownCommand.new "Unknown command #{command_code}"
  end
end

#handle_m0001(arg, options = {}) ⇒ Object



252
253
254
255
256
257
258
# File 'lib/rsmp/tlc/traffic_controller.rb', line 252

def handle_m0001 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  switch_functional_position arg['status'],
    timeout: arg['timeout'].to_i*60,
    source: 'forced'

end

#handle_m0002(arg, options = {}) ⇒ Object



260
261
262
263
264
265
266
267
# File 'lib/rsmp/tlc/traffic_controller.rb', line 260

def handle_m0002 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  if TrafficControllerSite.from_rsmp_bool(arg['status'])
    switch_plan arg['timeplan'], source: 'force'
  else
    switch_plan 0, source: 'startup'     # TODO use clock/calender
  end
end

#handle_m0003(arg, options = {}) ⇒ Object



269
270
271
272
# File 'lib/rsmp/tlc/traffic_controller.rb', line 269

def handle_m0003 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  switch_traffic_situation arg['traficsituation'], source: 'forced'
end

#handle_m0004(arg, options = {}) ⇒ Object



279
280
281
282
283
284
285
# File 'lib/rsmp/tlc/traffic_controller.rb', line 279

def handle_m0004 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  # don't restart immeediately, since we need to first send command response
  # instead, defer an action, which will be handled by the TLC site
  log "Sheduling restart of TLC", level: :info
  @node.defer :restart
end

#handle_m0005(arg, options = {}) ⇒ Object



287
288
289
290
291
292
293
294
295
296
297
# File 'lib/rsmp/tlc/traffic_controller.rb', line 287

def handle_m0005 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  @emergency_route = (arg['status'] == 'True')
  @emergency_route_number = arg['emergencyroute'].to_i

  if @emergency_route
    log "Switching to emergency route #{@emergency_route_number}", level: :info
  else
    log "Switching off emergency route", level: :info
  end
end

#handle_m0006(arg, options = {}) ⇒ Object



315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
# File 'lib/rsmp/tlc/traffic_controller.rb', line 315

def handle_m0006 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  input = arg['input'].to_i
  status = string_to_bool arg['status']
  unless input>=1 && input<=@inputs.size
    raise MessageRejected.new("Input must be in the range 1-#{@inputs.size}")
  end
  if status
    log "Activating input #{input}", level: :info
  else
    log "Deactivating input #{input}", level: :info
  end
  change = @inputs.set input, status
  input_logic input, change if change != nil
end

#handle_m0007(arg, options = {}) ⇒ Object



331
332
333
334
# File 'lib/rsmp/tlc/traffic_controller.rb', line 331

def handle_m0007 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  set_fixed_time_control arg['status'], source: 'forced'
end

#handle_m0012(arg, options = {}) ⇒ Object



336
337
338
# File 'lib/rsmp/tlc/traffic_controller.rb', line 336

def handle_m0012 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
end

#handle_m0013(arg, options = {}) ⇒ Object



340
341
342
# File 'lib/rsmp/tlc/traffic_controller.rb', line 340

def handle_m0013 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
end

#handle_m0014(arg, options = {}) ⇒ Object



350
351
352
353
354
355
356
357
358
359
360
# File 'lib/rsmp/tlc/traffic_controller.rb', line 350

def handle_m0014 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  plan = find_plan arg['plan']
  arg['status'].split(',').each do |item|
    matched = /(\d+)-(\d+)/.match item
    band = matched[1].to_i
    value = matched[2].to_i
    log "Set plan #{arg['plan']} dynamic band #{band} to #{value}", level: :info
    plan.set_band band, value
  end
end

#handle_m0015(arg, options = {}) ⇒ Object



362
363
364
# File 'lib/rsmp/tlc/traffic_controller.rb', line 362

def handle_m0015 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
end

#handle_m0016(arg, options = {}) ⇒ Object



366
367
368
# File 'lib/rsmp/tlc/traffic_controller.rb', line 366

def handle_m0016 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
end

#handle_m0017(arg, options = {}) ⇒ Object



370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
# File 'lib/rsmp/tlc/traffic_controller.rb', line 370

def handle_m0017 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  arg['status'].split(',').each do |item|
    elems = item.split('-')
    nr = elems[0].to_i
    plan = elems[1].to_i
    hour = elems[2].to_i
    min = elems[3].to_i
    if nr<0 || nr>12
      raise InvalidMessage.new "time table id must be between 0 and 12, got #{nr}"
    end
    #p "nr: #{nr}, plan #{plan} at #{hour}:#{min}"
    @day_time_table[nr] = {plan: plan, hour: hour, min:min}
  end
end

#handle_m0018(arg, options = {}) ⇒ Object



386
387
388
# File 'lib/rsmp/tlc/traffic_controller.rb', line 386

def handle_m0018 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
end

#handle_m0019(arg, options = {}) ⇒ Object



416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
# File 'lib/rsmp/tlc/traffic_controller.rb', line 416

def handle_m0019 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  input = arg['input'].to_i
  force = string_to_bool arg['status']
  forced_value = string_to_bool arg['inputValue']
  unless input>=1 && input<=@inputs.size
    raise MessageRejected.new("Input must be in the range 1-#{@inputs.size}")
  end
  if force
    log "Forcing input #{input} to #{forced_value}", level: :info
  else
    log "Releasing input #{input}", level: :info
  end
  change = @inputs.set_forcing input, force, forced_value
  input_logic input, change if change != nil
end

#handle_m0020(arg, options = {}) ⇒ Object



433
434
435
# File 'lib/rsmp/tlc/traffic_controller.rb', line 433

def handle_m0020 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
end

#handle_m0021(arg, options = {}) ⇒ Object



437
438
439
# File 'lib/rsmp/tlc/traffic_controller.rb', line 437

def handle_m0021 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
end

#handle_m0022(arg, options = {}) ⇒ Object



441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
# File 'lib/rsmp/tlc/traffic_controller.rb', line 441

def handle_m0022 arg, options={}
  id = arg['requestId']
  type = arg['type']
  priority = @signal_priorities.find { |priority| priority.id == id }
  case type
  when 'new'
    if priority
      raise MessageRejected.new("Priority Request #{id} already exists")
    else
      #ref = arg.slice('signalGroupId','inputId','connectionId','approachId','laneInId','laneOutId')
      if arg['signalGroupId']
        signal_group = node.find_component arg['signalGroupId']
      end

      level = arg['level']
      eta = arg['eta']
      vehicleType = arg['vehicleType']
      @signal_priorities << SignalPriority.new(node:self, id:id, level:level, eta:eta, vehicleType:vehicleType)
      log "Priority request for signal group #{signal_group.c_id} received with id #{id}", level: :info
    end          
  when 'update'
    if priority
      log "Priority Request #{id} updated", level: :info
    else
      raise MessageRejected.new("Cannot update priority request #{id}, not found")
    end
  when 'cancel'
    if priority
      @signal_priorities.delete priority
    else
      raise MessageRejected.new("Cannot cancel priority request #{id}, not found")
    end    
  else
    raise MessageRejected.new("Unknown type #{type}")
  end
end

#handle_m0023(arg, options = {}) ⇒ Object



478
479
480
481
482
483
484
485
486
487
488
489
490
# File 'lib/rsmp/tlc/traffic_controller.rb', line 478

def handle_m0023 arg, options={}
  @node.verify_security_code 2, arg['securityCode']
  timeout = arg['status'].to_i
  unless timeout>=0 and timeout <= 65535
    raise RSMP::MessageRejected.new "Timeout must be in the range 0-65535, got #{timeout}"
  end
  if timeout == 0
    log "Dynamic bands timeout disabled", level: :info
  else
    log "Dynamic bands timeout set to #{timeout}min", level: :info
  end
  @dynamic_bands_timeout = timeout
end

#handle_m0103(arg, options = {}) ⇒ Object



492
493
494
495
# File 'lib/rsmp/tlc/traffic_controller.rb', line 492

def handle_m0103 arg, options={}
  level = {'Level1'=>1,'Level2'=>2}[arg['status']]
  @node.change_security_code level, arg['oldSecurityCode'], arg['newSecurityCode']
end

#handle_m0104(arg, options = {}) ⇒ Object



497
498
499
500
501
502
503
504
505
506
507
508
509
510
# File 'lib/rsmp/tlc/traffic_controller.rb', line 497

def handle_m0104 arg, options={}
  @node.verify_security_code 1, arg['securityCode']
  time = Time.new(
    arg['year'],
    arg['month'],
    arg['day'],
    arg['hour'],
    arg['minute'],
    arg['second'],
    'UTC'
  )
  clock.set time
  log "Clock set to #{time}, (adjustment is #{clock.adjustment}s)", level: :info
end

#handle_s0001(status_code, status_name = nil, options = {}) ⇒ Object



571
572
573
574
575
576
577
578
579
580
581
582
# File 'lib/rsmp/tlc/traffic_controller.rb', line 571

def handle_s0001 status_code, status_name=nil, options={}
  case status_name
  when 'signalgroupstatus'
    TrafficControllerSite.make_status format_signal_group_status
  when 'cyclecounter'
    TrafficControllerSite.make_status @cycle_counter.to_s
  when 'basecyclecounter'
    TrafficControllerSite.make_status @cycle_counter.to_s
  when 'stage'
    TrafficControllerSite.make_status 0.to_s
  end
end

#handle_s0002(status_code, status_name = nil, options = {}) ⇒ Object



584
585
586
587
588
589
# File 'lib/rsmp/tlc/traffic_controller.rb', line 584

def handle_s0002 status_code, status_name=nil, options={}
  case status_name
  when 'detectorlogicstatus'
    TrafficControllerSite.make_status @detector_logics.map { |dl| bool_to_digit(dl.value) }.join
  end
end

#handle_s0003(status_code, status_name = nil, options = {}) ⇒ Object



591
592
593
594
595
596
597
598
# File 'lib/rsmp/tlc/traffic_controller.rb', line 591

def handle_s0003 status_code, status_name=nil, options={}
  case status_name
  when 'inputstatus'
    TrafficControllerSite.make_status @inputs.actual_string
  when 'extendedinputstatus'
    TrafficControllerSite.make_status 0.to_s
  end
end

#handle_s0004(status_code, status_name = nil, options = {}) ⇒ Object



600
601
602
603
604
605
606
607
# File 'lib/rsmp/tlc/traffic_controller.rb', line 600

def handle_s0004 status_code, status_name=nil, options={}
  case status_name
  when 'outputstatus'
    TrafficControllerSite.make_status 0
  when 'extendedoutputstatus'
    TrafficControllerSite.make_status 0
  end
end

#handle_s0005(status_code, status_name = nil, options = {}) ⇒ Object



609
610
611
612
613
614
# File 'lib/rsmp/tlc/traffic_controller.rb', line 609

def handle_s0005 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status @is_starting
  end
end

#handle_s0006(status_code, status_name = nil, options = {}) ⇒ Object



616
617
618
619
620
621
622
623
# File 'lib/rsmp/tlc/traffic_controller.rb', line 616

def handle_s0006 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status @emergency_route
  when 'emergencystage'
    TrafficControllerSite.make_status @emergency_route_number
  end
end

#handle_s0007(status_code, status_name = nil, options = {}) ⇒ Object



625
626
627
628
629
630
631
632
633
634
# File 'lib/rsmp/tlc/traffic_controller.rb', line 625

def handle_s0007 status_code, status_name=nil, options={}
  case status_name
  when 'intersection'
    TrafficControllerSite.make_status @intersection
  when 'status'
    TrafficControllerSite.make_status @function_position != 'Dark'
  when 'source'
    TrafficControllerSite.make_status @function_position_source
  end
end

#handle_s0008(status_code, status_name = nil, options = {}) ⇒ Object



636
637
638
639
640
641
642
643
644
645
# File 'lib/rsmp/tlc/traffic_controller.rb', line 636

def handle_s0008 status_code, status_name=nil, options={}
  case status_name
  when 'intersection'
    TrafficControllerSite.make_status @intersection
  when 'status'
    TrafficControllerSite.make_status @manual_control
  when 'source'
    TrafficControllerSite.make_status @manual_control_source
  end
end

#handle_s0009(status_code, status_name = nil, options = {}) ⇒ Object



647
648
649
650
651
652
653
654
655
656
# File 'lib/rsmp/tlc/traffic_controller.rb', line 647

def handle_s0009 status_code, status_name=nil, options={}
  case status_name
  when 'intersection'
    TrafficControllerSite.make_status @intersection
  when 'status'
    TrafficControllerSite.make_status @fixed_time_control
  when 'source'
    TrafficControllerSite.make_status @fixed_time_control_source
  end
end

#handle_s0010(status_code, status_name = nil, options = {}) ⇒ Object



658
659
660
661
662
663
664
665
666
667
# File 'lib/rsmp/tlc/traffic_controller.rb', line 658

def handle_s0010 status_code, status_name=nil, options={}
  case status_name
  when 'intersection'
    TrafficControllerSite.make_status @intersection
  when 'status'
    TrafficControllerSite.make_status @isolated_control
  when 'source'
    TrafficControllerSite.make_status @isolated_control_source
  end
end

#handle_s0011(status_code, status_name = nil, options = {}) ⇒ Object



669
670
671
672
673
674
675
676
677
678
# File 'lib/rsmp/tlc/traffic_controller.rb', line 669

def handle_s0011 status_code, status_name=nil, options={}
  case status_name
  when 'intersection'
    TrafficControllerSite.make_status @intersection
  when 'status'
    TrafficControllerSite.make_status TrafficControllerSite.to_rmsp_bool( @function_position == 'YellowFlash' )
  when 'source'
    TrafficControllerSite.make_status @function_position_source
  end
end

#handle_s0012(status_code, status_name = nil, options = {}) ⇒ Object



680
681
682
683
684
685
686
687
688
689
# File 'lib/rsmp/tlc/traffic_controller.rb', line 680

def handle_s0012 status_code, status_name=nil, options={}
  case status_name
  when 'intersection'
    TrafficControllerSite.make_status @intersection
  when 'status'
    TrafficControllerSite.make_status @all_red
  when 'source'
    TrafficControllerSite.make_status @all_red_source
  end
end

#handle_s0013(status_code, status_name = nil, options = {}) ⇒ Object



691
692
693
694
695
696
697
698
# File 'lib/rsmp/tlc/traffic_controller.rb', line 691

def handle_s0013 status_code, status_name=nil, options={}
  case status_name
  when 'intersection'
    TrafficControllerSite.make_status @intersection
  when 'status'
    TrafficControllerSite.make_status @police_key
  end
end

#handle_s0014(status_code, status_name = nil, options = {}) ⇒ Object



700
701
702
703
704
705
706
707
# File 'lib/rsmp/tlc/traffic_controller.rb', line 700

def handle_s0014 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status @plan
  when 'source'
    TrafficControllerSite.make_status @plan_source
  end
end

#handle_s0015(status_code, status_name = nil, options = {}) ⇒ Object



709
710
711
712
713
714
715
716
# File 'lib/rsmp/tlc/traffic_controller.rb', line 709

def handle_s0015 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status @traffic_situation
  when 'source'
    TrafficControllerSite.make_status @traffic_situation_source
  end
end

#handle_s0016(status_code, status_name = nil, options = {}) ⇒ Object



718
719
720
721
722
723
# File 'lib/rsmp/tlc/traffic_controller.rb', line 718

def handle_s0016 status_code, status_name=nil, options={}
  case status_name
  when 'number'
    TrafficControllerSite.make_status @detector_logics.size
  end
end

#handle_s0017(status_code, status_name = nil, options = {}) ⇒ Object



725
726
727
728
729
730
# File 'lib/rsmp/tlc/traffic_controller.rb', line 725

def handle_s0017 status_code, status_name=nil, options={}
  case status_name
  when 'number'
    TrafficControllerSite.make_status @signal_groups.size
  end
end

#handle_s0018(status_code, status_name = nil, options = {}) ⇒ Object



732
733
734
735
736
737
# File 'lib/rsmp/tlc/traffic_controller.rb', line 732

def handle_s0018 status_code, status_name=nil, options={}
  case status_name
  when 'number'
    TrafficControllerSite.make_status @plans.size
  end
end

#handle_s0019(status_code, status_name = nil, options = {}) ⇒ Object



739
740
741
742
743
744
# File 'lib/rsmp/tlc/traffic_controller.rb', line 739

def handle_s0019 status_code, status_name=nil, options={}
  case status_name
  when 'number'
    TrafficControllerSite.make_status @num_traffic_situations
  end
end

#handle_s0020(status_code, status_name = nil, options = {}) ⇒ Object



746
747
748
749
750
751
752
753
# File 'lib/rsmp/tlc/traffic_controller.rb', line 746

def handle_s0020 status_code, status_name=nil, options={}
  case status_name
  when 'intersection'
    TrafficControllerSite.make_status @intersection
  when 'controlmode'
    TrafficControllerSite.make_status @control_mode
  end
end

#handle_s0021(status_code, status_name = nil, options = {}) ⇒ Object



755
756
757
758
759
760
# File 'lib/rsmp/tlc/traffic_controller.rb', line 755

def handle_s0021 status_code, status_name=nil, options={}
  case status_name
  when 'detectorlogics'
    TrafficControllerSite.make_status @detector_logics.map { |logic| bool_to_digit(logic.forced)}.join
  end
end

#handle_s0022(status_code, status_name = nil, options = {}) ⇒ Object



762
763
764
765
766
767
# File 'lib/rsmp/tlc/traffic_controller.rb', line 762

def handle_s0022 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status @plans.keys.join(',')
  end
end

#handle_s0023(status_code, status_name = nil, options = {}) ⇒ Object



769
770
771
772
773
774
775
776
# File 'lib/rsmp/tlc/traffic_controller.rb', line 769

def handle_s0023 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    dynamic_bands = @plans.map { |nr,plan| plan.dynamic_bands_string }
    str = dynamic_bands.compact.join(',')
    TrafficControllerSite.make_status str
  end
end

#handle_s0024(status_code, status_name = nil, options = {}) ⇒ Object



778
779
780
781
782
783
# File 'lib/rsmp/tlc/traffic_controller.rb', line 778

def handle_s0024 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status '1-0'
  end
end

#handle_s0026(status_code, status_name = nil, options = {}) ⇒ Object



785
786
787
788
789
790
# File 'lib/rsmp/tlc/traffic_controller.rb', line 785

def handle_s0026 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status '0-00'
  end
end

#handle_s0027(status_code, status_name = nil, options = {}) ⇒ Object



792
793
794
795
796
797
798
799
800
# File 'lib/rsmp/tlc/traffic_controller.rb', line 792

def handle_s0027 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    status = @day_time_table.map do |i,item|
      "#{i}-#{item[:plan]}-#{item[:hour]}-#{item[:min]}"
    end.join(',')
    TrafficControllerSite.make_status status
  end
end

#handle_s0028(status_code, status_name = nil, options = {}) ⇒ Object



802
803
804
805
806
807
# File 'lib/rsmp/tlc/traffic_controller.rb', line 802

def handle_s0028 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status '00-00'
  end
end

#handle_s0029(status_code, status_name = nil, options = {}) ⇒ Object



809
810
811
812
813
814
# File 'lib/rsmp/tlc/traffic_controller.rb', line 809

def handle_s0029 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status @inputs.forced_string
  end
end

#handle_s0030(status_code, status_name = nil, options = {}) ⇒ Object



816
817
818
819
820
821
# File 'lib/rsmp/tlc/traffic_controller.rb', line 816

def handle_s0030 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status ''
  end
end

#handle_s0031(status_code, status_name = nil, options = {}) ⇒ Object



823
824
825
826
827
828
# File 'lib/rsmp/tlc/traffic_controller.rb', line 823

def handle_s0031 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status ''
  end
end

#handle_s0032(status_code, status_name = nil, options = {}) ⇒ Object



830
831
832
833
834
835
836
837
838
839
# File 'lib/rsmp/tlc/traffic_controller.rb', line 830

def handle_s0032 status_code, status_name=nil, options={}
  case status_name
  when 'intersection'
    TrafficControllerSite.make_status @intersection
  when 'status'
    TrafficControllerSite.make_status 'local'
  when 'source'
    TrafficControllerSite.make_status @intersection_source
  end
end

#handle_s0033(status_code, status_name = nil, options = {}) ⇒ Object



841
842
843
844
845
846
# File 'lib/rsmp/tlc/traffic_controller.rb', line 841

def handle_s0033 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status get_priority_list
  end
end

#handle_s0091(status_code, status_name = nil, options = {}) ⇒ Object



848
849
850
851
852
853
854
855
856
857
858
859
860
861
862
# File 'lib/rsmp/tlc/traffic_controller.rb', line 848

def handle_s0091 status_code, status_name=nil, options={}
  if Proxy.version_meets_requirement? options[:sxl_version], '>=1.1'
    case status_name
    when 'user'
      TrafficControllerSite.make_status 0
    end
  else
    case status_name
    when 'user'
      TrafficControllerSite.make_status 'nobody'
    when 'status'
      TrafficControllerSite.make_status 'logout'
    end
  end
end

#handle_s0092(status_code, status_name = nil, options = {}) ⇒ Object



864
865
866
867
868
869
870
871
872
873
874
875
876
877
878
# File 'lib/rsmp/tlc/traffic_controller.rb', line 864

def handle_s0092 status_code, status_name=nil, options={}
  if Proxy.version_meets_requirement? options[:sxl_version], '>=1.1'
    case status_name
    when 'user'
      TrafficControllerSite.make_status 0
    end
  else
    case status_name
    when 'user'
      TrafficControllerSite.make_status 'nobody'
    when 'status'
      TrafficControllerSite.make_status 'logout'
    end
  end
end

#handle_s0095(status_code, status_name = nil, options = {}) ⇒ Object



880
881
882
883
884
885
# File 'lib/rsmp/tlc/traffic_controller.rb', line 880

def handle_s0095 status_code, status_name=nil, options={}
  case status_name
  when 'status'
    TrafficControllerSite.make_status RSMP::VERSION
  end
end

#handle_s0096(status_code, status_name = nil, options = {}) ⇒ Object



887
888
889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
# File 'lib/rsmp/tlc/traffic_controller.rb', line 887

def handle_s0096 status_code, status_name=nil, options={}
  now = clock.now
  case status_name
  when 'year'
    TrafficControllerSite.make_status now.year.to_s.rjust(4, "0")
  when 'month'
    TrafficControllerSite.make_status now.month.to_s.rjust(2, "0")
  when 'day'
    TrafficControllerSite.make_status now.day.to_s.rjust(2, "0")
  when 'hour'
    TrafficControllerSite.make_status now.hour.to_s.rjust(2, "0")
  when 'minute'
    TrafficControllerSite.make_status now.min.to_s.rjust(2, "0")
  when 'second'
    TrafficControllerSite.make_status now.sec.to_s.rjust(2, "0")
  end
end

#handle_s0097(status_code, status_name = nil, options = {}) ⇒ Object



905
906
907
908
909
910
911
912
913
# File 'lib/rsmp/tlc/traffic_controller.rb', line 905

def handle_s0097 status_code, status_name=nil, options={}
  case status_name
  when 'checksum'
    TrafficControllerSite.make_status '1'
  when 'timestamp'
    now = clock.to_s
    TrafficControllerSite.make_status now
  end
end

#handle_s0205(status_code, status_name = nil, options = {}) ⇒ Object



915
916
917
918
919
920
921
922
# File 'lib/rsmp/tlc/traffic_controller.rb', line 915

def handle_s0205 status_code, status_name=nil, options={}
  case status_name
  when 'start'
    TrafficControllerSite.make_status clock.to_s
  when 'vehicles'
    TrafficControllerSite.make_status 0
  end
end

#handle_s0206(status_code, status_name = nil, options = {}) ⇒ Object



924
925
926
927
928
929
930
931
# File 'lib/rsmp/tlc/traffic_controller.rb', line 924

def handle_s0206 status_code, status_name=nil, options={}
  case status_name
  when 'start'
    TrafficControllerSite.make_status clock.to_s
  when 'speed'
    TrafficControllerSite.make_status 0
  end
end

#handle_s0207(status_code, status_name = nil, options = {}) ⇒ Object



933
934
935
936
937
938
939
940
# File 'lib/rsmp/tlc/traffic_controller.rb', line 933

def handle_s0207 status_code, status_name=nil, options={}
  case status_name
  when 'start'
    TrafficControllerSite.make_status clock.to_s
  when 'occupancy'
    TrafficControllerSite.make_status 0
  end
end

#handle_s0208(status_code, status_name = nil, options = {}) ⇒ Object



942
943
944
945
946
947
948
949
950
951
952
953
954
955
956
957
958
959
960
961
962
963
964
965
# File 'lib/rsmp/tlc/traffic_controller.rb', line 942

def handle_s0208 status_code, status_name=nil, options={}
  case status_name
  when 'start'
    TrafficControllerSite.make_status clock.to_s
  when 'P'
    TrafficControllerSite.make_status 0
  when 'PS'
    TrafficControllerSite.make_status 0
  when 'L'
    TrafficControllerSite.make_status 0
  when 'LS'
    TrafficControllerSite.make_status 0
  when 'B'
    TrafficControllerSite.make_status 0
  when 'SP'
    TrafficControllerSite.make_status 0
  when 'MC'
    TrafficControllerSite.make_status 0
  when 'C'
    TrafficControllerSite.make_status 0
  when 'F'
    TrafficControllerSite.make_status 0
  end
end

#initiate_startup_sequenceObject



164
165
166
167
168
169
170
# File 'lib/rsmp/tlc/traffic_controller.rb', line 164

def initiate_startup_sequence
  log "Initiating startup sequence", level: :info
  reset_modes
  @startup_sequence_active = true
  @startup_sequence_initiated_at = nil
  @startup_sequence_pos = nil
end

#input_logic(input, change) ⇒ Object



299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
# File 'lib/rsmp/tlc/traffic_controller.rb', line 299

def input_logic input, change
  return unless @input_programming && change != nil
  actions = @input_programming[input]
  return unless actions
  if actions['raise']
    alarm_code = actions['raise']
    if change
      log "Activating input #{input} is programmed to activate alarm #{alarm_code}", level: :info
      activate_alarm alarm_code
    else
      log "Deactivating input #{input} is programmed to deactivate alarm #{alarm_code}", level: :info
      deactivate_alarm alarm_code
    end
  end
end

#move_cycle_counterObject



144
145
146
147
# File 'lib/rsmp/tlc/traffic_controller.rb', line 144

def move_cycle_counter
  counter = Time.now.to_i % @cycle_time
  @cycle_counter = counter
end

#move_startup_sequenceObject



178
179
180
181
182
183
184
185
186
187
188
189
# File 'lib/rsmp/tlc/traffic_controller.rb', line 178

def move_startup_sequence
  was = @startup_sequence_pos
  if @startup_sequence_initiated_at == nil
    @startup_sequence_initiated_at = Time.now.to_i + 1
    @startup_sequence_pos = 0
  else
    @startup_sequence_pos = Time.now.to_i - @startup_sequence_initiated_at
  end
  if @startup_sequence_pos >= @startup_sequence.size
    end_startup_sequence
  end
end

#normal_control?Boolean

Returns:

  • (Boolean)


83
84
85
# File 'lib/rsmp/tlc/traffic_controller.rb', line 83

def normal_control?
  @function_position == 'NormalControl'
end

#output_statesObject



191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
# File 'lib/rsmp/tlc/traffic_controller.rb', line 191

def output_states
  return unless @live_output

  str = @signal_groups.map do |group|
    state = group.state
    s = "#{group.c_id}:#{state}"
    if state =~ /^[1-9]$/
        s.colorize(:green)
    elsif state =~ /^[NOP]$/
      s.colorize(:yellow)
    elsif state =~ /^[ae]$/
      s.colorize(:light_black)
    elsif state =~ /^[f]$/
      s.colorize(:yellow)
    elsif state =~ /^[g]$/
      s.colorize(:red)
    else
      s.colorize(:red)
    end
  end.join ' '

  modes = '.'*9
  modes[0] = 'N' if @function_position == 'NormalControl'
  modes[1] = 'Y' if @function_position == 'YellowFlash'
  modes[2] = 'D' if @function_position == 'Dark'
  modes[3] = 'B' if @booting
  modes[4] = 'S' if @startup_sequence_active
  modes[5] = 'M' if @manual_control
  modes[6] = 'F' if @fixed_time_control
  modes[7] = 'R' if @all_red
  modes[8] = 'I' if @isolated_control
  modes[9] = 'P' if @police_key != 0

  plan = "P#{@plan}"

  # create folders if needed
  FileUtils.mkdir_p File.dirname(@live_output)

  # append a line with the current state to the file
  File.open @live_output, 'w' do |file|
    file.puts "#{modes}  #{plan.rjust(2)}  #{@cycle_counter.to_s.rjust(3)}  #{str}\r"
  end
end

#prune_prioritiesObject



128
129
130
131
132
# File 'lib/rsmp/tlc/traffic_controller.rb', line 128

def prune_priorities
  # TODO spec states that update must be send one time when it reaches the state 'completed',
  # and then be removed. so we need to know when it has been sent
  @signal_priorities.delete_if {|priority| priority.state=='completed' && priority.age >= 1.5 }
end

#resetObject



54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
# File 'lib/rsmp/tlc/traffic_controller.rb', line 54

def reset
  reset_modes
  @cycle_counter = 0
  @plan = 1
  @plan_source = 'startup'
  @intersection = 0
  @intersection_source = 'startup'
  @emergency_route = false
  @emergency_route_number = 0
  @traffic_situation = 0
  @traffic_situation_source = 'startup'
  @day_time_table = {}
  @startup_sequence_active = false
  @startup_sequence_initiated_at = nil
  @startup_sequence_pos = 0
  @time_int = nil
  @inputs.reset
  @signal_priorities = []
  @dynamic_bands_timeout = 0
end

#reset_modesObject



34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
# File 'lib/rsmp/tlc/traffic_controller.rb', line 34

def reset_modes
  @function_position = 'NormalControl'
  @function_position_source = 'startup'
  @previous_functional_position = nil
  @functional_position_timeout = nil

  @booting = false
  @is_starting = false
  @control_mode = 'control'
  @manual_control = false
  @manual_control_source = 'startup'
  @fixed_time_control = false
  @fixed_time_control_source = 'startup'
  @isolated_control = false
  @isolated_control_source = 'startup'
  @all_red = false
  @all_red_source = 'startup'
  @police_key = 0
end

#set_fixed_time_control(status, source:) ⇒ Object



517
518
519
520
# File 'lib/rsmp/tlc/traffic_controller.rb', line 517

def set_fixed_time_control status, source:
  @fixed_time_control = status
  @fixed_time_control_source = source
end

#set_input(i, value) ⇒ Object



512
513
514
515
# File 'lib/rsmp/tlc/traffic_controller.rb', line 512

def set_input i, value
  return unless i>=0 && i<@num_inputs
  @inputs[i] = bool_to_digit arg['value']
end

#signal_priority_changed(priority, state) ⇒ Object



124
125
126
# File 'lib/rsmp/tlc/traffic_controller.rb', line 124

def signal_priority_changed priority, state
  #puts "priority #{priority.id} -> #{state}"
end

#startup_stateObject



158
159
160
161
162
# File 'lib/rsmp/tlc/traffic_controller.rb', line 158

def startup_state
  return unless @startup_sequence_active
  return unless @startup_sequence_pos
  @startup_sequence[ @startup_sequence_pos ]
end

#string_to_bool(bool_str) ⇒ Object



390
391
392
393
394
395
396
397
398
399
# File 'lib/rsmp/tlc/traffic_controller.rb', line 390

def string_to_bool bool_str
  case bool_str
    when 'True'
      true
    when 'False'
      false
    else
      raise RSMP::MessageRejected.new "Invalid boolean '#{bool}', must be 'True' or 'False'"
  end
end

#switch_functional_position(mode, timeout: nil, reverting: false, source:) ⇒ Object



534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
# File 'lib/rsmp/tlc/traffic_controller.rb', line 534

def switch_functional_position mode, timeout: nil, reverting: false, source:
  unless ['NormalControl','YellowFlash','Dark'].include? mode
    raise RSMP::MessageRejected.new "Invalid functional position '#{mode}', must be NormalControl, YellowFlash or Dark'"
  end
  if reverting
    log "Reverting to functional position #{mode} after timeout", level: :info
  elsif timeout && timeout > 0
    log "Switching to functional position #{mode} with timeout #{(timeout/60).round(1)}min", level: :info
    @previous_functional_position = @function_position
    now = clock.now
    @functional_position_timeout = now + timeout
  else
    log "Switching to functional position #{mode}", level: :info
  end 
  if mode == 'NormalControl'
    initiate_startup_sequence if @function_position != 'NormalControl'
  end
  @function_position = mode
  @function_position_source = source
  mode
end

#switch_plan(plan, source:) ⇒ Object



522
523
524
525
526
527
528
529
530
531
532
# File 'lib/rsmp/tlc/traffic_controller.rb', line 522

def switch_plan plan, source:
  plan_nr = plan.to_i
  if plan_nr == 0
    log "Switching to plan selection by time table", level: :info
  else
    plan = find_plan plan_nr
    log "Switching to plan #{plan_nr}", level: :info
  end
  @plan = plan_nr
  @plan_source = source
end

#switch_traffic_situation(situation, source:) ⇒ Object



274
275
276
277
# File 'lib/rsmp/tlc/traffic_controller.rb', line 274

def switch_traffic_situation situation, source:
  @traffic_situation = situation.to_i
  @traffic_situation_source = 'forced'
end

#timer(now) ⇒ Object



108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
# File 'lib/rsmp/tlc/traffic_controller.rb', line 108

def timer now
  # TODO use monotone timer, to avoid jumps in case the user sets the system time
  time = Time.now.to_i
  return if time == @time_int
  @time_int = time
  move_cycle_counter
  check_functional_position_timeout
  move_startup_sequence if @startup_sequence_active

  @signal_groups.each { |group| group.timer }
  @signal_priorities.each {|priority| priority.timer }

  output_states
  prune_priorities
end

#yellow_flash?Boolean

Returns:

  • (Boolean)


79
80
81
# File 'lib/rsmp/tlc/traffic_controller.rb', line 79

def yellow_flash?
  @function_position == 'YellowFlash'
end