Class: RSMP::TLC::TrafficController
- Inherits:
-
Component
- Object
- ComponentBase
- Component
- RSMP::TLC::TrafficController
- Defined in:
- lib/rsmp/tlc/traffic_controller.rb
Overview
TrafficController is the main component of a TrafficControllerSite. It handles all command and status for the main component, and keeps track of signal plans, detector logics, inputs, etc. which do not have dedicated components.
Constant Summary
Constants inherited from ComponentBase
ComponentBase::AGGREGATED_STATUS_KEYS
Instance Attribute Summary collapse
-
#cycle_counter ⇒ Object
readonly
Returns the value of attribute cycle_counter.
-
#cycle_time ⇒ Object
readonly
Returns the value of attribute cycle_time.
-
#functional_position ⇒ Object
readonly
Returns the value of attribute functional_position.
-
#plan ⇒ Object
readonly
Returns the value of attribute plan.
-
#pos ⇒ Object
readonly
Returns the value of attribute pos.
-
#startup_sequence ⇒ Object
readonly
Returns the value of attribute startup_sequence.
-
#startup_sequence_active ⇒ Object
readonly
Returns the value of attribute startup_sequence_active.
-
#startup_sequence_pos ⇒ Object
readonly
Returns the value of attribute startup_sequence_pos.
Attributes inherited from ComponentBase
#aggregated_status, #aggregated_status_bools, #alarms, #c_id, #grouped, #node, #ntsOId, #statuses, #xNId
Instance Method Summary collapse
- #add_detector_logic(logic) ⇒ Object
- #add_signal_group(group) ⇒ Object
- #bool_string_to_digit(bool) ⇒ Object
- #bool_to_digit(bool) ⇒ Object
- #check_functional_position_timeout ⇒ Object
- #clock ⇒ Object
- #current_plan ⇒ Object
- #dark? ⇒ Boolean
- #end_startup_sequence ⇒ Object
- #find_plan(plan_nr) ⇒ Object
- #format_signal_group_status ⇒ Object
- #get_status(code, name = nil) ⇒ Object
- #handle_command(command_code, arg) ⇒ Object
- #handle_m0001(arg) ⇒ Object
- #handle_m0002(arg) ⇒ Object
- #handle_m0003(arg) ⇒ Object
- #handle_m0004(arg) ⇒ Object
- #handle_m0005(arg) ⇒ Object
- #handle_m0006(arg) ⇒ Object
- #handle_m0007(arg) ⇒ Object
- #handle_m0012(arg) ⇒ Object
- #handle_m0013(arg) ⇒ Object
- #handle_m0014(arg) ⇒ Object
- #handle_m0015(arg) ⇒ Object
- #handle_m0016(arg) ⇒ Object
- #handle_m0017(arg) ⇒ Object
- #handle_m0018(arg) ⇒ Object
- #handle_m0019(arg) ⇒ Object
- #handle_m0020(arg) ⇒ Object
- #handle_m0021(arg) ⇒ Object
- #handle_m0103(arg) ⇒ Object
- #handle_m0104(arg) ⇒ Object
- #handle_s0001(status_code, status_name = nil) ⇒ Object
- #handle_s0002(status_code, status_name = nil) ⇒ Object
- #handle_s0003(status_code, status_name = nil) ⇒ Object
- #handle_s0004(status_code, status_name = nil) ⇒ Object
- #handle_s0005(status_code, status_name = nil) ⇒ Object
- #handle_s0006(status_code, status_name = nil) ⇒ Object
- #handle_s0007(status_code, status_name = nil) ⇒ Object
- #handle_s0008(status_code, status_name = nil) ⇒ Object
- #handle_s0009(status_code, status_name = nil) ⇒ Object
- #handle_s0010(status_code, status_name = nil) ⇒ Object
- #handle_s0011(status_code, status_name = nil) ⇒ Object
- #handle_s0012(status_code, status_name = nil) ⇒ Object
- #handle_s0013(status_code, status_name = nil) ⇒ Object
- #handle_s0014(status_code, status_name = nil) ⇒ Object
- #handle_s0015(status_code, status_name = nil) ⇒ Object
- #handle_s0016(status_code, status_name = nil) ⇒ Object
- #handle_s0017(status_code, status_name = nil) ⇒ Object
- #handle_s0018(status_code, status_name = nil) ⇒ Object
- #handle_s0019(status_code, status_name = nil) ⇒ Object
- #handle_s0020(status_code, status_name = nil) ⇒ Object
- #handle_s0021(status_code, status_name = nil) ⇒ Object
- #handle_s0022(status_code, status_name = nil) ⇒ Object
- #handle_s0023(status_code, status_name = nil) ⇒ Object
- #handle_s0024(status_code, status_name = nil) ⇒ Object
- #handle_s0026(status_code, status_name = nil) ⇒ Object
- #handle_s0027(status_code, status_name = nil) ⇒ Object
- #handle_s0028(status_code, status_name = nil) ⇒ Object
- #handle_s0029(status_code, status_name = nil) ⇒ Object
- #handle_s0030(status_code, status_name = nil) ⇒ Object
- #handle_s0031(status_code, status_name = nil) ⇒ Object
- #handle_s0091(status_code, status_name = nil) ⇒ Object
- #handle_s0092(status_code, status_name = nil) ⇒ Object
- #handle_s0095(status_code, status_name = nil) ⇒ Object
- #handle_s0096(status_code, status_name = nil) ⇒ Object
- #handle_s0097(status_code, status_name = nil) ⇒ Object
- #handle_s0205(status_code, status_name = nil) ⇒ Object
- #handle_s0206(status_code, status_name = nil) ⇒ Object
- #handle_s0207(status_code, status_name = nil) ⇒ Object
- #handle_s0208(status_code, status_name = nil) ⇒ Object
-
#initialize(node:, id:, ntsOId: nil, xNId: nil, cycle_time: 10, signal_plans:, startup_sequence:, live_output: nil, inputs: {}) ⇒ TrafficController
constructor
A new instance of TrafficController.
- #initiate_startup_sequence ⇒ Object
- #input_logic(input, change) ⇒ Object
- #move_cycle_counter ⇒ Object
- #move_startup_sequence ⇒ Object
- #normal_control? ⇒ Boolean
- #output_states ⇒ Object
- #reset ⇒ Object
- #reset_modes ⇒ Object
- #set_fixed_time_control(status) ⇒ Object
- #set_input(i, value) ⇒ Object
- #startup_state ⇒ Object
- #string_to_bool(bool_str) ⇒ Object
- #switch_functional_position(mode, timeout: nil, reverting: false) ⇒ Object
- #switch_plan(plan) ⇒ Object
- #timer(now) ⇒ Object
- #yellow_flash? ⇒ Boolean
Methods inherited from Component
#activate_alarm, #deactivate_alarm, #resume_alarm, #suspend_alarm
Methods inherited from ComponentBase
#aggregated_status_changed, #clear_aggregated_status, #get_alarm_state, #log, #set_aggregated_status, #set_aggregated_status_bools
Methods included from Inspect
Constructor Details
#initialize(node:, id:, ntsOId: nil, xNId: nil, cycle_time: 10, signal_plans:, startup_sequence:, live_output: nil, inputs: {}) ⇒ TrafficController
Returns a new instance of TrafficController.
12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 12 def initialize node:, id:, ntsOId: nil, xNId: nil, cycle_time: 10, signal_plans:, startup_sequence:, live_output:nil, inputs:{} super node: node, id: id, ntsOId: ntsOId, xNId: xNId, grouped: true @signal_groups = [] @detector_logics = [] @plans = signal_plans @cycle_time = cycle_time @num_traffic_situations = 1 if inputs num_inputs = inputs['total'] @input_programming = inputs['programming'] else @input_programming = nil end @inputs = TLC::Inputs.new num_inputs || 8 @startup_sequence = startup_sequence @live_output = live_output reset end |
Instance Attribute Details
#cycle_counter ⇒ Object (readonly)
Returns the value of attribute cycle_counter.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def cycle_counter @cycle_counter end |
#cycle_time ⇒ Object (readonly)
Returns the value of attribute cycle_time.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def cycle_time @cycle_time end |
#functional_position ⇒ Object (readonly)
Returns the value of attribute functional_position.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def functional_position @functional_position end |
#plan ⇒ Object (readonly)
Returns the value of attribute plan.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def plan @plan end |
#pos ⇒ Object (readonly)
Returns the value of attribute pos.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def pos @pos end |
#startup_sequence ⇒ Object (readonly)
Returns the value of attribute startup_sequence.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def startup_sequence @startup_sequence end |
#startup_sequence_active ⇒ Object (readonly)
Returns the value of attribute startup_sequence_active.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def startup_sequence_active @startup_sequence_active end |
#startup_sequence_pos ⇒ Object (readonly)
Returns the value of attribute startup_sequence_pos.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def startup_sequence_pos @startup_sequence_pos end |
Instance Method Details
#add_detector_logic(logic) ⇒ Object
94 95 96 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 94 def add_detector_logic logic @detector_logics << logic end |
#add_signal_group(group) ⇒ Object
90 91 92 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 90 def add_signal_group group @signal_groups << group end |
#bool_string_to_digit(bool) ⇒ Object
361 362 363 364 365 366 367 368 369 370 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 361 def bool_string_to_digit bool case bool when 'True' '1' when 'False' '0' else raise RSMP::MessageRejected.new "Invalid boolean '#{bool}', must be 'True' or 'False'" end end |
#bool_to_digit(bool) ⇒ Object
372 373 374 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 372 def bool_to_digit bool bool ? '1' : '0' end |
#check_functional_position_timeout ⇒ Object
117 118 119 120 121 122 123 124 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 117 def check_functional_position_timeout return unless @functional_position_timeout if clock.now >= @functional_position_timeout switch_functional_position @previous_functional_position, reverting: true @functional_position_timeout = nil @previous_functional_position = nil end end |
#clock ⇒ Object
77 78 79 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 77 def clock node.clock end |
#current_plan ⇒ Object
81 82 83 84 85 86 87 88 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 81 def current_plan # TODO plan 0 should means use time table if @plans @plans[ plan ] || @plans.values.first else nil end end |
#dark? ⇒ Boolean
65 66 67 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 65 def dark? @function_position == 'Dark' end |
#end_startup_sequence ⇒ Object
140 141 142 143 144 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 140 def end_startup_sequence @startup_sequence_active = false @startup_sequence_initiated_at = nil @startup_sequence_pos = nil end |
#find_plan(plan_nr) ⇒ Object
304 305 306 307 308 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 304 def find_plan plan_nr plan = @plans[plan_nr.to_i] raise InvalidMessage.new "unknown signal plan #{plan_nr}, known only [#{@plans.keys.join(',')}]" unless plan plan end |
#format_signal_group_status ⇒ Object
203 204 205 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 203 def format_signal_group_status @signal_groups.map { |group| group.state }.join end |
#get_status(code, name = nil) ⇒ Object
462 463 464 465 466 467 468 469 470 471 472 473 474 475 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 462 def get_status code, name=nil case code when 'S0001', 'S0002', 'S0003', 'S0004', 'S0005', 'S0006', 'S0007', 'S0008', 'S0009', 'S0010', 'S0011', 'S0012', 'S0013', 'S0014', 'S0015', 'S0016', 'S0017', 'S0018', 'S0019', 'S0020', 'S0021', 'S0022', 'S0023', 'S0024', 'S0026', 'S0027', 'S0028', 'S0029', 'S0030', 'S0031', 'S0091', 'S0092', 'S0095', 'S0096', 'S0097', 'S0205', 'S0206', 'S0207', 'S0208' return send("handle_#{code.downcase}", code, name) else raise InvalidMessage.new "unknown status code #{code}" end end |
#handle_command(command_code, arg) ⇒ Object
207 208 209 210 211 212 213 214 215 216 217 218 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 207 def handle_command command_code, arg case command_code when 'M0001', 'M0002', 'M0003', 'M0004', 'M0005', 'M0006', 'M0007', 'M0012', 'M0013', 'M0014', 'M0015', 'M0016', 'M0017', 'M0018', 'M0019', 'M0020', 'M0021', 'M0022', 'M0103', 'M0104' return send("handle_#{command_code.downcase}", arg) else raise UnknownCommand.new "Unknown command #{command_code}" end end |
#handle_m0001(arg) ⇒ Object
220 221 222 223 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 220 def handle_m0001 arg @node.verify_security_code 2, arg['securityCode'] switch_functional_position arg['status'], timeout: arg['timeout'].to_i*60 end |
#handle_m0002(arg) ⇒ Object
225 226 227 228 229 230 231 232 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 225 def handle_m0002 arg @node.verify_security_code 2, arg['securityCode'] if TrafficControllerSite.from_rsmp_bool(arg['status']) switch_plan arg['timeplan'] else switch_plan 0 # TODO use clock/calender end end |
#handle_m0003(arg) ⇒ Object
234 235 236 237 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 234 def handle_m0003 arg @node.verify_security_code 2, arg['securityCode'] @traffic_situation = arg['traficsituation'].to_i end |
#handle_m0004(arg) ⇒ Object
239 240 241 242 243 244 245 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 239 def handle_m0004 arg @node.verify_security_code 2, arg['securityCode'] # don't restart immeediately, since we need to first send command response # instead, defer an action, which will be handled by the TLC site log "Sheduling restart of TLC", level: :info @node.defer :restart end |
#handle_m0005(arg) ⇒ Object
247 248 249 250 251 252 253 254 255 256 257 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 247 def handle_m0005 arg @node.verify_security_code 2, arg['securityCode'] @emergency_route = (arg['status'] == 'True') @emergency_route_number = arg['emergencyroute'].to_i if @emergency_route log "Switching to emergency route #{@emergency_route_number}", level: :info else log "Switching off emergency route", level: :info end end |
#handle_m0006(arg) ⇒ Object
275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 275 def handle_m0006 arg @node.verify_security_code 2, arg['securityCode'] input = arg['input'].to_i status = string_to_bool arg['status'] unless input>=1 && input<=@inputs.size raise MessageRejected.new("Input must be in the range 1-#{@inputs.size}") end if status log "Activating input #{input}", level: :info else log "Deactivating input #{input}", level: :info end change = @inputs.set input, status input_logic input, change if change != nil end |
#handle_m0007(arg) ⇒ Object
291 292 293 294 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 291 def handle_m0007 arg @node.verify_security_code 2, arg['securityCode'] set_fixed_time_control arg['status'] end |
#handle_m0012(arg) ⇒ Object
296 297 298 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 296 def handle_m0012 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0013(arg) ⇒ Object
300 301 302 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 300 def handle_m0013 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0014(arg) ⇒ Object
310 311 312 313 314 315 316 317 318 319 320 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 310 def handle_m0014 arg @node.verify_security_code 2, arg['securityCode'] plan = find_plan arg['plan'] arg['status'].split(',').each do |item| matched = /(\d+)-(\d+)/.match item band = matched[1].to_i value = matched[2].to_i log "Set plan #{arg['plan']} dynamic band #{band} to #{value}", level: :info plan.set_band band, value end end |
#handle_m0015(arg) ⇒ Object
322 323 324 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 322 def handle_m0015 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0016(arg) ⇒ Object
326 327 328 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 326 def handle_m0016 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0017(arg) ⇒ Object
330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 330 def handle_m0017 arg @node.verify_security_code 2, arg['securityCode'] arg['status'].split(',').each do |item| elems = item.split('-') nr = elems[0].to_i plan = elems[1].to_i hour = elems[2].to_i min = elems[3].to_i if nr<0 || nr>12 raise InvalidMessage.new "time table id must be between 0 and 12, got #{nr}" end #p "nr: #{nr}, plan #{plan} at #{hour}:#{min}" @day_time_table[nr] = {plan: plan, hour: hour, min:min} end end |
#handle_m0018(arg) ⇒ Object
346 347 348 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 346 def handle_m0018 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0019(arg) ⇒ Object
376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 376 def handle_m0019 arg @node.verify_security_code 2, arg['securityCode'] input = arg['input'].to_i force = string_to_bool arg['status'] forced_value = string_to_bool arg['inputValue'] unless input>=1 && input<=@inputs.size raise MessageRejected.new("Input must be in the range 1-#{@inputs.size}") end if force log "Forcing input #{input} to #{forced_value}", level: :info else log "Releasing input #{input}", level: :info end change = @inputs.set_forcing input, force, forced_value input_logic input, change if change != nil end |
#handle_m0020(arg) ⇒ Object
393 394 395 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 393 def handle_m0020 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0021(arg) ⇒ Object
397 398 399 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 397 def handle_m0021 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0103(arg) ⇒ Object
401 402 403 404 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 401 def handle_m0103 arg level = {'Level1'=>1,'Level2'=>2}[arg['status']] @node.change_security_code level, arg['oldSecurityCode'], arg['newSecurityCode'] end |
#handle_m0104(arg) ⇒ Object
406 407 408 409 410 411 412 413 414 415 416 417 418 419 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 406 def handle_m0104 arg @node.verify_security_code 1, arg['securityCode'] time = Time.new( arg['year'], arg['month'], arg['day'], arg['hour'], arg['minute'], arg['second'], 'UTC' ) clock.set time log "Clock set to #{time}, (adjustment is #{clock.adjustment}s)", level: :info end |
#handle_s0001(status_code, status_name = nil) ⇒ Object
477 478 479 480 481 482 483 484 485 486 487 488 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 477 def handle_s0001 status_code, status_name=nil case status_name when 'signalgroupstatus' TrafficControllerSite.make_status format_signal_group_status when 'cyclecounter' TrafficControllerSite.make_status @cycle_counter.to_s when 'basecyclecounter' TrafficControllerSite.make_status @cycle_counter.to_s when 'stage' TrafficControllerSite.make_status 0.to_s end end |
#handle_s0002(status_code, status_name = nil) ⇒ Object
490 491 492 493 494 495 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 490 def handle_s0002 status_code, status_name=nil case status_name when 'detectorlogicstatus' TrafficControllerSite.make_status @detector_logics.map { |dl| bool_to_digit(dl.value) }.join end end |
#handle_s0003(status_code, status_name = nil) ⇒ Object
497 498 499 500 501 502 503 504 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 497 def handle_s0003 status_code, status_name=nil case status_name when 'inputstatus' TrafficControllerSite.make_status @inputs.actual_string when 'extendedinputstatus' TrafficControllerSite.make_status 0.to_s end end |
#handle_s0004(status_code, status_name = nil) ⇒ Object
506 507 508 509 510 511 512 513 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 506 def handle_s0004 status_code, status_name=nil case status_name when 'outputstatus' TrafficControllerSite.make_status 0 when 'extendedoutputstatus' TrafficControllerSite.make_status 0 end end |
#handle_s0005(status_code, status_name = nil) ⇒ Object
515 516 517 518 519 520 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 515 def handle_s0005 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @is_starting end end |
#handle_s0006(status_code, status_name = nil) ⇒ Object
522 523 524 525 526 527 528 529 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 522 def handle_s0006 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @emergency_route when 'emergencystage' TrafficControllerSite.make_status @emergency_route_number end end |
#handle_s0007(status_code, status_name = nil) ⇒ Object
531 532 533 534 535 536 537 538 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 531 def handle_s0007 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @function_position != 'Dark' end end |
#handle_s0008(status_code, status_name = nil) ⇒ Object
540 541 542 543 544 545 546 547 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 540 def handle_s0008 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @manual_control end end |
#handle_s0009(status_code, status_name = nil) ⇒ Object
549 550 551 552 553 554 555 556 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 549 def handle_s0009 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @fixed_time_control end end |
#handle_s0010(status_code, status_name = nil) ⇒ Object
558 559 560 561 562 563 564 565 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 558 def handle_s0010 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @isolated_control end end |
#handle_s0011(status_code, status_name = nil) ⇒ Object
567 568 569 570 571 572 573 574 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 567 def handle_s0011 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status TrafficControllerSite.to_rmsp_bool( @function_position == 'YellowFlash' ) end end |
#handle_s0012(status_code, status_name = nil) ⇒ Object
576 577 578 579 580 581 582 583 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 576 def handle_s0012 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @all_red end end |
#handle_s0013(status_code, status_name = nil) ⇒ Object
585 586 587 588 589 590 591 592 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 585 def handle_s0013 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @police_key end end |
#handle_s0014(status_code, status_name = nil) ⇒ Object
594 595 596 597 598 599 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 594 def handle_s0014 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @plan end end |
#handle_s0015(status_code, status_name = nil) ⇒ Object
601 602 603 604 605 606 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 601 def handle_s0015 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @traffic_situation end end |
#handle_s0016(status_code, status_name = nil) ⇒ Object
608 609 610 611 612 613 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 608 def handle_s0016 status_code, status_name=nil case status_name when 'number' TrafficControllerSite.make_status @detector_logics.size end end |
#handle_s0017(status_code, status_name = nil) ⇒ Object
615 616 617 618 619 620 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 615 def handle_s0017 status_code, status_name=nil case status_name when 'number' TrafficControllerSite.make_status @signal_groups.size end end |
#handle_s0018(status_code, status_name = nil) ⇒ Object
622 623 624 625 626 627 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 622 def handle_s0018 status_code, status_name=nil case status_name when 'number' TrafficControllerSite.make_status @plans.size end end |
#handle_s0019(status_code, status_name = nil) ⇒ Object
629 630 631 632 633 634 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 629 def handle_s0019 status_code, status_name=nil case status_name when 'number' TrafficControllerSite.make_status @num_traffic_situations end end |
#handle_s0020(status_code, status_name = nil) ⇒ Object
636 637 638 639 640 641 642 643 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 636 def handle_s0020 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'controlmode' TrafficControllerSite.make_status @control_mode end end |
#handle_s0021(status_code, status_name = nil) ⇒ Object
645 646 647 648 649 650 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 645 def handle_s0021 status_code, status_name=nil case status_name when 'detectorlogics' TrafficControllerSite.make_status @detector_logics.map { |logic| bool_to_digit(logic.forced)}.join end end |
#handle_s0022(status_code, status_name = nil) ⇒ Object
652 653 654 655 656 657 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 652 def handle_s0022 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @plans.keys.join(',') end end |
#handle_s0023(status_code, status_name = nil) ⇒ Object
659 660 661 662 663 664 665 666 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 659 def handle_s0023 status_code, status_name=nil case status_name when 'status' dynamic_bands = @plans.map { |nr,plan| plan.dynamic_bands_string } str = dynamic_bands.compact.join(',') TrafficControllerSite.make_status str end end |
#handle_s0024(status_code, status_name = nil) ⇒ Object
668 669 670 671 672 673 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 668 def handle_s0024 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '1-0' end end |
#handle_s0026(status_code, status_name = nil) ⇒ Object
675 676 677 678 679 680 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 675 def handle_s0026 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '0-00' end end |
#handle_s0027(status_code, status_name = nil) ⇒ Object
682 683 684 685 686 687 688 689 690 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 682 def handle_s0027 status_code, status_name=nil case status_name when 'status' status = @day_time_table.map do |i,item| "#{i}-#{item[:plan]}-#{item[:hour]}-#{item[:min]}" end.join(',') TrafficControllerSite.make_status status end end |
#handle_s0028(status_code, status_name = nil) ⇒ Object
692 693 694 695 696 697 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 692 def handle_s0028 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '00-00' end end |
#handle_s0029(status_code, status_name = nil) ⇒ Object
699 700 701 702 703 704 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 699 def handle_s0029 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @inputs.forced_string end end |
#handle_s0030(status_code, status_name = nil) ⇒ Object
706 707 708 709 710 711 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 706 def handle_s0030 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '' end end |
#handle_s0031(status_code, status_name = nil) ⇒ Object
713 714 715 716 717 718 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 713 def handle_s0031 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '' end end |
#handle_s0091(status_code, status_name = nil) ⇒ Object
720 721 722 723 724 725 726 727 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 720 def handle_s0091 status_code, status_name=nil case status_name when 'user' TrafficControllerSite.make_status 'nobody' when 'status' TrafficControllerSite.make_status 'logout' end end |
#handle_s0092(status_code, status_name = nil) ⇒ Object
729 730 731 732 733 734 735 736 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 729 def handle_s0092 status_code, status_name=nil case status_name when 'user' TrafficControllerSite.make_status 'nobody' when 'status' TrafficControllerSite.make_status 'logout' end end |
#handle_s0095(status_code, status_name = nil) ⇒ Object
738 739 740 741 742 743 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 738 def handle_s0095 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status RSMP::VERSION end end |
#handle_s0096(status_code, status_name = nil) ⇒ Object
745 746 747 748 749 750 751 752 753 754 755 756 757 758 759 760 761 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 745 def handle_s0096 status_code, status_name=nil now = clock.now case status_name when 'year' TrafficControllerSite.make_status now.year.to_s.rjust(4, "0") when 'month' TrafficControllerSite.make_status now.month.to_s.rjust(2, "0") when 'day' TrafficControllerSite.make_status now.day.to_s.rjust(2, "0") when 'hour' TrafficControllerSite.make_status now.hour.to_s.rjust(2, "0") when 'minute' TrafficControllerSite.make_status now.min.to_s.rjust(2, "0") when 'second' TrafficControllerSite.make_status now.sec.to_s.rjust(2, "0") end end |
#handle_s0097(status_code, status_name = nil) ⇒ Object
763 764 765 766 767 768 769 770 771 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 763 def handle_s0097 status_code, status_name=nil case status_name when 'checksum' TrafficControllerSite.make_status '1' when 'timestamp' now = clock.to_s TrafficControllerSite.make_status now end end |
#handle_s0205(status_code, status_name = nil) ⇒ Object
773 774 775 776 777 778 779 780 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 773 def handle_s0205 status_code, status_name=nil case status_name when 'start' TrafficControllerSite.make_status clock.to_s when 'vehicles' TrafficControllerSite.make_status 0 end end |
#handle_s0206(status_code, status_name = nil) ⇒ Object
782 783 784 785 786 787 788 789 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 782 def handle_s0206 status_code, status_name=nil case status_name when 'start' TrafficControllerSite.make_status clock.to_s when 'speed' TrafficControllerSite.make_status 0 end end |
#handle_s0207(status_code, status_name = nil) ⇒ Object
791 792 793 794 795 796 797 798 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 791 def handle_s0207 status_code, status_name=nil case status_name when 'start' TrafficControllerSite.make_status clock.to_s when 'occupancy' TrafficControllerSite.make_status 0 end end |
#handle_s0208(status_code, status_name = nil) ⇒ Object
800 801 802 803 804 805 806 807 808 809 810 811 812 813 814 815 816 817 818 819 820 821 822 823 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 800 def handle_s0208 status_code, status_name=nil case status_name when 'start' TrafficControllerSite.make_status clock.to_s when 'P' TrafficControllerSite.make_status 0 when 'PS' TrafficControllerSite.make_status 0 when 'L' TrafficControllerSite.make_status 0 when 'LS' TrafficControllerSite.make_status 0 when 'B' TrafficControllerSite.make_status 0 when 'SP' TrafficControllerSite.make_status 0 when 'MC' TrafficControllerSite.make_status 0 when 'C' TrafficControllerSite.make_status 0 when 'F' TrafficControllerSite.make_status 0 end end |
#initiate_startup_sequence ⇒ Object
132 133 134 135 136 137 138 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 132 def initiate_startup_sequence log "Initiating startup sequence", level: :info reset_modes @startup_sequence_active = true @startup_sequence_initiated_at = nil @startup_sequence_pos = nil end |
#input_logic(input, change) ⇒ Object
259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 259 def input_logic input, change return unless @input_programming && change != nil actions = @input_programming[input] return unless actions if actions['raise'] alarm_code = actions['raise'] if change log "Activating input #{input} is programmed to activate alarm #{alarm_code}", level: :info activate_alarm alarm_code else log "Deactivating input #{input} is programmed to deactivate alarm #{alarm_code}", level: :info deactivate_alarm alarm_code end end end |
#move_cycle_counter ⇒ Object
112 113 114 115 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 112 def move_cycle_counter counter = Time.now.to_i % @cycle_time @cycle_counter = counter end |
#move_startup_sequence ⇒ Object
146 147 148 149 150 151 152 153 154 155 156 157 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 146 def move_startup_sequence was = @startup_sequence_pos if @startup_sequence_initiated_at == nil @startup_sequence_initiated_at = Time.now.to_i + 1 @startup_sequence_pos = 0 else @startup_sequence_pos = Time.now.to_i - @startup_sequence_initiated_at end if @startup_sequence_pos >= @startup_sequence.size end_startup_sequence end end |
#normal_control? ⇒ Boolean
73 74 75 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 73 def normal_control? @function_position == 'NormalControl' end |
#output_states ⇒ Object
159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 159 def output_states return unless @live_output str = @signal_groups.map do |group| state = group.state s = "#{group.c_id}:#{state}" if state =~ /^[1-9]$/ s.colorize(:green) elsif state =~ /^[NOP]$/ s.colorize(:yellow) elsif state =~ /^[ae]$/ s.colorize(:light_black) elsif state =~ /^[f]$/ s.colorize(:yellow) elsif state =~ /^[g]$/ s.colorize(:red) else s.colorize(:red) end end.join ' ' modes = '.'*9 modes[0] = 'N' if @function_position == 'NormalControl' modes[1] = 'Y' if @function_position == 'YellowFlash' modes[2] = 'D' if @function_position == 'Dark' modes[3] = 'B' if @booting modes[4] = 'S' if @startup_sequence_active modes[5] = 'M' if @manual_control modes[6] = 'F' if @fixed_time_control modes[7] = 'R' if @all_red modes[8] = 'I' if @isolated_control modes[9] = 'P' if @police_key != 0 plan = "P#{@plan}" # create folders if needed FileUtils.mkdir_p File.dirname(@live_output) # append a line with the current state to the file File.open @live_output, 'w' do |file| file.puts "#{modes} #{plan.rjust(2)} #{@cycle_counter.to_s.rjust(3)} #{str}\r" end end |
#reset ⇒ Object
49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 49 def reset reset_modes @cycle_counter = 0 @plan = 1 @intersection = 0 @emergency_route = false @emergency_route_number = 0 @traffic_situation = 0 @day_time_table = {} @startup_sequence_active = false @startup_sequence_initiated_at = nil @startup_sequence_pos = 0 @time_int = nil @inputs.reset end |
#reset_modes ⇒ Object
34 35 36 37 38 39 40 41 42 43 44 45 46 47 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 34 def reset_modes @function_position = 'NormalControl' @previous_functional_position = nil @functional_position_timeout = nil @booting = false @is_starting = false @control_mode = 'control' @manual_control = false @fixed_time_control = false @isolated_control = false @all_red = false @police_key = 0 end |
#set_fixed_time_control(status) ⇒ Object
426 427 428 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 426 def set_fixed_time_control status @fixed_time_control = status end |
#set_input(i, value) ⇒ Object
421 422 423 424 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 421 def set_input i, value return unless i>=0 && i<@num_inputs @inputs[i] = bool_to_digit arg['value'] end |
#startup_state ⇒ Object
126 127 128 129 130 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 126 def startup_state return unless @startup_sequence_active return unless @startup_sequence_pos @startup_sequence[ @startup_sequence_pos ] end |
#string_to_bool(bool_str) ⇒ Object
350 351 352 353 354 355 356 357 358 359 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 350 def string_to_bool bool_str case bool_str when 'True' true when 'False' false else raise RSMP::MessageRejected.new "Invalid boolean '#{bool}', must be 'True' or 'False'" end end |
#switch_functional_position(mode, timeout: nil, reverting: false) ⇒ Object
441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 460 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 441 def switch_functional_position mode, timeout: nil, reverting: false unless ['NormalControl','YellowFlash','Dark'].include? mode raise RSMP::MessageRejected.new "Invalid functional position '#{mode}', must be NormalControl, YellowFlash or Dark'" end if reverting log "Reverting to functional position #{mode} after timeout", level: :info elsif timeout && timeout > 0 log "Switching to functional position #{mode} with timeout #{(timeout/60).round(1)}min", level: :info @previous_functional_position = @function_position now = clock.now @functional_position_timeout = now + timeout else log "Switching to functional position #{mode}", level: :info end if mode == 'NormalControl' initiate_startup_sequence if @function_position != 'NormalControl' end @function_position = mode mode end |
#switch_plan(plan) ⇒ Object
430 431 432 433 434 435 436 437 438 439 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 430 def switch_plan plan plan_nr = plan.to_i if plan_nr == 0 log "Switching to plan selection by time table", level: :info else plan = find_plan plan_nr log "Switching to plan #{plan_nr}", level: :info end @plan = plan_nr end |
#timer(now) ⇒ Object
98 99 100 101 102 103 104 105 106 107 108 109 110 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 98 def timer now # TODO use monotone timer, to avoid jumps in case the user sets the system time time = Time.now.to_i return if time == @time_int @time_int = time move_cycle_counter check_functional_position_timeout move_startup_sequence if @startup_sequence_active @signal_groups.each { |group| group.timer } output_states end |
#yellow_flash? ⇒ Boolean
69 70 71 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 69 def yellow_flash? @function_position == 'YellowFlash' end |