Class: RSMP::TLC::TrafficController
- Defined in:
- lib/rsmp/tlc/traffic_controller.rb
Overview
TrafficController is the main component of a TrafficControllerSite. It handles all command and status for the main component, and keeps track of signal plans, detector logics, inputs, etc. which do not have dedicated components.
Constant Summary
Constants inherited from Component
Component::AGGREGATED_STATUS_KEYS
Instance Attribute Summary collapse
-
#cycle_counter ⇒ Object
readonly
Returns the value of attribute cycle_counter.
-
#cycle_time ⇒ Object
readonly
Returns the value of attribute cycle_time.
-
#dark_mode ⇒ Object
readonly
Returns the value of attribute dark_mode.
-
#plan ⇒ Object
readonly
Returns the value of attribute plan.
-
#pos ⇒ Object
readonly
Returns the value of attribute pos.
-
#startup_sequence ⇒ Object
readonly
Returns the value of attribute startup_sequence.
-
#startup_sequence_active ⇒ Object
readonly
Returns the value of attribute startup_sequence_active.
-
#startup_sequence_pos ⇒ Object
readonly
Returns the value of attribute startup_sequence_pos.
-
#yellow_flash ⇒ Object
readonly
Returns the value of attribute yellow_flash.
Attributes inherited from Component
#aggregated_status, #aggregated_status_bools, #alarms, #c_id, #grouped, #node, #statuses
Instance Method Summary collapse
- #add_detector_logic(logic) ⇒ Object
- #add_signal_group(group) ⇒ Object
- #clock ⇒ Object
- #current_plan ⇒ Object
- #end_startup_sequence ⇒ Object
- #find_plan(plan_nr) ⇒ Object
- #format_signal_group_status ⇒ Object
- #get_status(code, name = nil) ⇒ Object
- #handle_command(command_code, arg) ⇒ Object
- #handle_m0001(arg) ⇒ Object
- #handle_m0002(arg) ⇒ Object
- #handle_m0003(arg) ⇒ Object
- #handle_m0004(arg) ⇒ Object
- #handle_m0005(arg) ⇒ Object
- #handle_m0006(arg) ⇒ Object
- #handle_m0007(arg) ⇒ Object
- #handle_m0012(arg) ⇒ Object
- #handle_m0013(arg) ⇒ Object
- #handle_m0014(arg) ⇒ Object
- #handle_m0015(arg) ⇒ Object
- #handle_m0016(arg) ⇒ Object
- #handle_m0017(arg) ⇒ Object
- #handle_m0018(arg) ⇒ Object
- #handle_m0019(arg) ⇒ Object
- #handle_m0020(arg) ⇒ Object
- #handle_m0021(arg) ⇒ Object
- #handle_m0103(arg) ⇒ Object
- #handle_m0104(arg) ⇒ Object
- #handle_s0001(status_code, status_name = nil) ⇒ Object
- #handle_s0002(status_code, status_name = nil) ⇒ Object
- #handle_s0003(status_code, status_name = nil) ⇒ Object
- #handle_s0004(status_code, status_name = nil) ⇒ Object
- #handle_s0005(status_code, status_name = nil) ⇒ Object
- #handle_s0006(status_code, status_name = nil) ⇒ Object
- #handle_s0007(status_code, status_name = nil) ⇒ Object
- #handle_s0008(status_code, status_name = nil) ⇒ Object
- #handle_s0009(status_code, status_name = nil) ⇒ Object
- #handle_s0010(status_code, status_name = nil) ⇒ Object
- #handle_s0011(status_code, status_name = nil) ⇒ Object
- #handle_s0012(status_code, status_name = nil) ⇒ Object
- #handle_s0013(status_code, status_name = nil) ⇒ Object
- #handle_s0014(status_code, status_name = nil) ⇒ Object
- #handle_s0015(status_code, status_name = nil) ⇒ Object
- #handle_s0016(status_code, status_name = nil) ⇒ Object
- #handle_s0017(status_code, status_name = nil) ⇒ Object
- #handle_s0018(status_code, status_name = nil) ⇒ Object
- #handle_s0019(status_code, status_name = nil) ⇒ Object
- #handle_s0020(status_code, status_name = nil) ⇒ Object
- #handle_s0021(status_code, status_name = nil) ⇒ Object
- #handle_s0022(status_code, status_name = nil) ⇒ Object
- #handle_s0023(status_code, status_name = nil) ⇒ Object
- #handle_s0024(status_code, status_name = nil) ⇒ Object
- #handle_s0026(status_code, status_name = nil) ⇒ Object
- #handle_s0027(status_code, status_name = nil) ⇒ Object
- #handle_s0028(status_code, status_name = nil) ⇒ Object
- #handle_s0029(status_code, status_name = nil) ⇒ Object
- #handle_s0030(status_code, status_name = nil) ⇒ Object
- #handle_s0031(status_code, status_name = nil) ⇒ Object
- #handle_s0091(status_code, status_name = nil) ⇒ Object
- #handle_s0092(status_code, status_name = nil) ⇒ Object
- #handle_s0095(status_code, status_name = nil) ⇒ Object
- #handle_s0096(status_code, status_name = nil) ⇒ Object
- #handle_s0097(status_code, status_name = nil) ⇒ Object
- #handle_s0205(status_code, status_name = nil) ⇒ Object
- #handle_s0206(status_code, status_name = nil) ⇒ Object
- #handle_s0207(status_code, status_name = nil) ⇒ Object
- #handle_s0208(status_code, status_name = nil) ⇒ Object
-
#initialize(node:, id:, cycle_time: 10, signal_plans:, startup_sequence:, live_output: nil) ⇒ TrafficController
constructor
A new instance of TrafficController.
- #initiate_startup_sequence ⇒ Object
- #move_cycle_counter ⇒ Object
- #move_startup_sequence ⇒ Object
- #output_states ⇒ Object
- #reset ⇒ Object
- #reset_modes ⇒ Object
- #set_fixed_time_control(status) ⇒ Object
- #set_input(i, value) ⇒ Object
- #startup_state ⇒ Object
- #switch_mode(mode) ⇒ Object
- #switch_plan(plan) ⇒ Object
- #timer(now) ⇒ Object
Methods inherited from Component
#aggregated_status_changed, #clear_aggregated_status, #handle_alarm, #handle_status_response, #handle_status_subscribe, #handle_status_unsubscribe, #handle_status_update, #log, #set_aggregated_status, #set_aggregated_status_bools, #store_status
Methods included from Inspect
Constructor Details
#initialize(node:, id:, cycle_time: 10, signal_plans:, startup_sequence:, live_output: nil) ⇒ TrafficController
Returns a new instance of TrafficController.
12 13 14 15 16 17 18 19 20 21 22 23 24 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 12 def initialize node:, id:, cycle_time: 10, signal_plans:, startup_sequence:, live_output:nil super node: node, id: id, grouped: true @signal_groups = [] @detector_logics = [] @plans = signal_plans @cycle_time = cycle_time @num_traffic_situations = 1 @num_inputs = 8 @startup_sequence = startup_sequence @live_output = live_output reset end |
Instance Attribute Details
#cycle_counter ⇒ Object (readonly)
Returns the value of attribute cycle_counter.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def cycle_counter @cycle_counter end |
#cycle_time ⇒ Object (readonly)
Returns the value of attribute cycle_time.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def cycle_time @cycle_time end |
#dark_mode ⇒ Object (readonly)
Returns the value of attribute dark_mode.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def dark_mode @dark_mode end |
#plan ⇒ Object (readonly)
Returns the value of attribute plan.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def plan @plan end |
#pos ⇒ Object (readonly)
Returns the value of attribute pos.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def pos @pos end |
#startup_sequence ⇒ Object (readonly)
Returns the value of attribute startup_sequence.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def startup_sequence @startup_sequence end |
#startup_sequence_active ⇒ Object (readonly)
Returns the value of attribute startup_sequence_active.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def startup_sequence_active @startup_sequence_active end |
#startup_sequence_pos ⇒ Object (readonly)
Returns the value of attribute startup_sequence_pos.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def startup_sequence_pos @startup_sequence_pos end |
#yellow_flash ⇒ Object (readonly)
Returns the value of attribute yellow_flash.
8 9 10 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 8 def yellow_flash @yellow_flash end |
Instance Method Details
#add_detector_logic(logic) ⇒ Object
78 79 80 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 78 def add_detector_logic logic @detector_logics << logic end |
#add_signal_group(group) ⇒ Object
74 75 76 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 74 def add_signal_group group @signal_groups << group end |
#clock ⇒ Object
61 62 63 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 61 def clock node.clock end |
#current_plan ⇒ Object
65 66 67 68 69 70 71 72 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 65 def current_plan # TODO plan 0 should means use time table if @plans @plans[ plan ] || @plans.values.first else nil end end |
#end_startup_sequence ⇒ Object
115 116 117 118 119 120 121 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 115 def end_startup_sequence @startup_sequence_active = false @startup_sequence_initiated_at = nil @startup_sequence_pos = nil @yellow_flash = false @dark_mode = false end |
#find_plan(plan_nr) ⇒ Object
259 260 261 262 263 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 259 def find_plan plan_nr plan = @plans[plan_nr.to_i] raise InvalidMessage.new "unknown signal plan #{plan_nr}, known only [#{@plans.keys.join(',')}]" unless plan plan end |
#format_signal_group_status ⇒ Object
175 176 177 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 175 def format_signal_group_status @signal_groups.map { |group| group.state }.join end |
#get_status(code, name = nil) ⇒ Object
374 375 376 377 378 379 380 381 382 383 384 385 386 387 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 374 def get_status code, name=nil case code when 'S0001', 'S0002', 'S0003', 'S0004', 'S0005', 'S0006', 'S0007', 'S0008', 'S0009', 'S0010', 'S0011', 'S0012', 'S0013', 'S0014', 'S0015', 'S0016', 'S0017', 'S0018', 'S0019', 'S0020', 'S0021', 'S0022', 'S0023', 'S0024', 'S0026', 'S0027', 'S0028', 'S0029', 'S0030', 'S0031', 'S0091', 'S0092', 'S0095', 'S0096', 'S0097', 'S0205', 'S0206', 'S0207', 'S0208' return send("handle_#{code.downcase}", code, name) else raise InvalidMessage.new "unknown status code #{code}" end end |
#handle_command(command_code, arg) ⇒ Object
179 180 181 182 183 184 185 186 187 188 189 190 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 179 def handle_command command_code, arg case command_code when 'M0001', 'M0002', 'M0003', 'M0004', 'M0005', 'M0006', 'M0007', 'M0012', 'M0013', 'M0014', 'M0015', 'M0016', 'M0017', 'M0018', 'M0019', 'M0020', 'M0021', 'M0022', 'M0103', 'M0104' return send("handle_#{command_code.downcase}", arg) else raise UnknownCommand.new "Unknown command #{command_code}" end end |
#handle_m0001(arg) ⇒ Object
192 193 194 195 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 192 def handle_m0001 arg @node.verify_security_code 2, arg['securityCode'] switch_mode arg['status'] end |
#handle_m0002(arg) ⇒ Object
197 198 199 200 201 202 203 204 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 197 def handle_m0002 arg @node.verify_security_code 2, arg['securityCode'] if TrafficControllerSite.from_rsmp_bool(arg['status']) switch_plan arg['timeplan'] else switch_plan 0 # TODO use clock/calender end end |
#handle_m0003(arg) ⇒ Object
206 207 208 209 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 206 def handle_m0003 arg @node.verify_security_code 2, arg['securityCode'] @traffic_situation = arg['traficsituation'].to_i end |
#handle_m0004(arg) ⇒ Object
211 212 213 214 215 216 217 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 211 def handle_m0004 arg @node.verify_security_code 2, arg['securityCode'] # don't restart immeediately, since we need to first send command response # instead, defer an action, which will be handled by the TLC site log "Sheduling restart of TLC", level: :info @node.defer :restart end |
#handle_m0005(arg) ⇒ Object
219 220 221 222 223 224 225 226 227 228 229 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 219 def handle_m0005 arg @node.verify_security_code 2, arg['securityCode'] @emergency_route = (arg['status'] == 'True') @emergency_route_number = arg['emergencyroute'].to_i if @emergency_route log "Switching to emergency route #{@emergency_route_number}", level: :info else log "Switching off emergency route", level: :info end end |
#handle_m0006(arg) ⇒ Object
231 232 233 234 235 236 237 238 239 240 241 242 243 244 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 231 def handle_m0006 arg @node.verify_security_code 2, arg['securityCode'] input = arg['input'].to_i idx = input - 1 return unless idx>=0 && input<@num_inputs # TODO should NotAck @input_activations[idx] = (arg['status']=='True' ? '1' : '0') result = @input_activations[idx]=='1' || @inputs[idx]=='1' @input_results[idx] = (result ? '1' : '0') if @input_activations[idx] log "Activate input #{idx}", level: :info else log "Deactivate input #{idx}", level: :info end end |
#handle_m0007(arg) ⇒ Object
246 247 248 249 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 246 def handle_m0007 arg @node.verify_security_code 2, arg['securityCode'] set_fixed_time_control arg['status'] end |
#handle_m0012(arg) ⇒ Object
251 252 253 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 251 def handle_m0012 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0013(arg) ⇒ Object
255 256 257 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 255 def handle_m0013 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0014(arg) ⇒ Object
265 266 267 268 269 270 271 272 273 274 275 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 265 def handle_m0014 arg @node.verify_security_code 2, arg['securityCode'] plan = find_plan arg['plan'] arg['status'].split(',').each do |item| matched = /(\d+)-(\d+)/.match item band = matched[1].to_i value = matched[2].to_i log "Set plan #{arg['plan']} dynamic band #{band} to #{value}", level: :info plan.set_band band, value end end |
#handle_m0015(arg) ⇒ Object
277 278 279 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 277 def handle_m0015 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0016(arg) ⇒ Object
281 282 283 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 281 def handle_m0016 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0017(arg) ⇒ Object
285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 285 def handle_m0017 arg @node.verify_security_code 2, arg['securityCode'] arg['status'].split(',').each do |item| elems = item.split('-') nr = elems[0].to_i plan = elems[1].to_i hour = elems[2].to_i min = elems[3].to_i if nr<0 || nr>12 raise InvalidMessage.new "time table id must be between 0 and 12, got #{nr}" end #p "nr: #{nr}, plan #{plan} at #{hour}:#{min}" @day_time_table[nr] = {plan: plan, hour: hour, min:min} end end |
#handle_m0018(arg) ⇒ Object
301 302 303 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 301 def handle_m0018 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0019(arg) ⇒ Object
305 306 307 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 305 def handle_m0019 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0020(arg) ⇒ Object
309 310 311 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 309 def handle_m0020 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0021(arg) ⇒ Object
313 314 315 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 313 def handle_m0021 arg @node.verify_security_code 2, arg['securityCode'] end |
#handle_m0103(arg) ⇒ Object
317 318 319 320 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 317 def handle_m0103 arg level = {'Level1'=>1,'Level2'=>2}[arg['status']] @node.change_security_code level, arg['oldSecurityCode'], arg['newSecurityCode'] end |
#handle_m0104(arg) ⇒ Object
322 323 324 325 326 327 328 329 330 331 332 333 334 335 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 322 def handle_m0104 arg @node.verify_security_code 1, arg['securityCode'] time = Time.new( arg['year'], arg['month'], arg['day'], arg['hour'], arg['minute'], arg['second'], 'UTC' ) @node.clock.set time log "Clock set to #{time}, (adjustment is #{@node.clock.adjustment}s)", level: :info end |
#handle_s0001(status_code, status_name = nil) ⇒ Object
389 390 391 392 393 394 395 396 397 398 399 400 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 389 def handle_s0001 status_code, status_name=nil case status_name when 'signalgroupstatus' TrafficControllerSite.make_status format_signal_group_status when 'cyclecounter' TrafficControllerSite.make_status @cycle_counter.to_s when 'basecyclecounter' TrafficControllerSite.make_status @cycle_counter.to_s when 'stage' TrafficControllerSite.make_status 0.to_s end end |
#handle_s0002(status_code, status_name = nil) ⇒ Object
402 403 404 405 406 407 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 402 def handle_s0002 status_code, status_name=nil case status_name when 'detectorlogicstatus' TrafficControllerSite.make_status @detector_logics.map { |dl| dl.value ? '1' : '0' }.join end end |
#handle_s0003(status_code, status_name = nil) ⇒ Object
409 410 411 412 413 414 415 416 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 409 def handle_s0003 status_code, status_name=nil case status_name when 'inputstatus' TrafficControllerSite.make_status @input_results when 'extendedinputstatus' TrafficControllerSite.make_status 0.to_s end end |
#handle_s0004(status_code, status_name = nil) ⇒ Object
418 419 420 421 422 423 424 425 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 418 def handle_s0004 status_code, status_name=nil case status_name when 'outputstatus' TrafficControllerSite.make_status 0 when 'extendedoutputstatus' TrafficControllerSite.make_status 0 end end |
#handle_s0005(status_code, status_name = nil) ⇒ Object
427 428 429 430 431 432 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 427 def handle_s0005 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @is_starting end end |
#handle_s0006(status_code, status_name = nil) ⇒ Object
434 435 436 437 438 439 440 441 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 434 def handle_s0006 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @emergency_route when 'emergencystage' TrafficControllerSite.make_status @emergency_route_number end end |
#handle_s0007(status_code, status_name = nil) ⇒ Object
443 444 445 446 447 448 449 450 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 443 def handle_s0007 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status !@dark_mode end end |
#handle_s0008(status_code, status_name = nil) ⇒ Object
452 453 454 455 456 457 458 459 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 452 def handle_s0008 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @manual_control end end |
#handle_s0009(status_code, status_name = nil) ⇒ Object
461 462 463 464 465 466 467 468 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 461 def handle_s0009 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @fixed_time_control end end |
#handle_s0010(status_code, status_name = nil) ⇒ Object
470 471 472 473 474 475 476 477 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 470 def handle_s0010 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @isolated_control end end |
#handle_s0011(status_code, status_name = nil) ⇒ Object
479 480 481 482 483 484 485 486 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 479 def handle_s0011 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @yellow_flash end end |
#handle_s0012(status_code, status_name = nil) ⇒ Object
488 489 490 491 492 493 494 495 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 488 def handle_s0012 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @all_red end end |
#handle_s0013(status_code, status_name = nil) ⇒ Object
497 498 499 500 501 502 503 504 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 497 def handle_s0013 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'status' TrafficControllerSite.make_status @police_key end end |
#handle_s0014(status_code, status_name = nil) ⇒ Object
506 507 508 509 510 511 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 506 def handle_s0014 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @plan end end |
#handle_s0015(status_code, status_name = nil) ⇒ Object
513 514 515 516 517 518 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 513 def handle_s0015 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @traffic_situation end end |
#handle_s0016(status_code, status_name = nil) ⇒ Object
520 521 522 523 524 525 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 520 def handle_s0016 status_code, status_name=nil case status_name when 'number' TrafficControllerSite.make_status @detector_logics.size end end |
#handle_s0017(status_code, status_name = nil) ⇒ Object
527 528 529 530 531 532 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 527 def handle_s0017 status_code, status_name=nil case status_name when 'number' TrafficControllerSite.make_status @signal_groups.size end end |
#handle_s0018(status_code, status_name = nil) ⇒ Object
534 535 536 537 538 539 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 534 def handle_s0018 status_code, status_name=nil case status_name when 'number' TrafficControllerSite.make_status @plans.size end end |
#handle_s0019(status_code, status_name = nil) ⇒ Object
541 542 543 544 545 546 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 541 def handle_s0019 status_code, status_name=nil case status_name when 'number' TrafficControllerSite.make_status @num_traffic_situations end end |
#handle_s0020(status_code, status_name = nil) ⇒ Object
548 549 550 551 552 553 554 555 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 548 def handle_s0020 status_code, status_name=nil case status_name when 'intersection' TrafficControllerSite.make_status @intersection when 'controlmode' TrafficControllerSite.make_status @control_mode end end |
#handle_s0021(status_code, status_name = nil) ⇒ Object
557 558 559 560 561 562 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 557 def handle_s0021 status_code, status_name=nil case status_name when 'detectorlogics' TrafficControllerSite.make_status @detector_logics.map { |logic| logic.forced=='True' ? '1' : '0'}.join end end |
#handle_s0022(status_code, status_name = nil) ⇒ Object
564 565 566 567 568 569 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 564 def handle_s0022 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status @plans.keys.join(',') end end |
#handle_s0023(status_code, status_name = nil) ⇒ Object
571 572 573 574 575 576 577 578 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 571 def handle_s0023 status_code, status_name=nil case status_name when 'status' dynamic_bands = @plans.map { |nr,plan| plan.dynamic_bands_string } str = dynamic_bands.compact.join(',') TrafficControllerSite.make_status str end end |
#handle_s0024(status_code, status_name = nil) ⇒ Object
580 581 582 583 584 585 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 580 def handle_s0024 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '1-0' end end |
#handle_s0026(status_code, status_name = nil) ⇒ Object
587 588 589 590 591 592 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 587 def handle_s0026 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '0-00' end end |
#handle_s0027(status_code, status_name = nil) ⇒ Object
594 595 596 597 598 599 600 601 602 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 594 def handle_s0027 status_code, status_name=nil case status_name when 'status' status = @day_time_table.map do |i,item| "#{i}-#{item[:plan]}-#{item[:hour]}-#{item[:min]}" end.join(',') TrafficControllerSite.make_status status end end |
#handle_s0028(status_code, status_name = nil) ⇒ Object
604 605 606 607 608 609 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 604 def handle_s0028 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '00-00' end end |
#handle_s0029(status_code, status_name = nil) ⇒ Object
611 612 613 614 615 616 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 611 def handle_s0029 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '' end end |
#handle_s0030(status_code, status_name = nil) ⇒ Object
618 619 620 621 622 623 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 618 def handle_s0030 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '' end end |
#handle_s0031(status_code, status_name = nil) ⇒ Object
625 626 627 628 629 630 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 625 def handle_s0031 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status '' end end |
#handle_s0091(status_code, status_name = nil) ⇒ Object
632 633 634 635 636 637 638 639 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 632 def handle_s0091 status_code, status_name=nil case status_name when 'user' TrafficControllerSite.make_status 'nobody' when 'status' TrafficControllerSite.make_status 'logout' end end |
#handle_s0092(status_code, status_name = nil) ⇒ Object
641 642 643 644 645 646 647 648 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 641 def handle_s0092 status_code, status_name=nil case status_name when 'user' TrafficControllerSite.make_status 'nobody' when 'status' TrafficControllerSite.make_status 'logout' end end |
#handle_s0095(status_code, status_name = nil) ⇒ Object
650 651 652 653 654 655 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 650 def handle_s0095 status_code, status_name=nil case status_name when 'status' TrafficControllerSite.make_status RSMP::VERSION end end |
#handle_s0096(status_code, status_name = nil) ⇒ Object
657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 657 def handle_s0096 status_code, status_name=nil now = clock.now case status_name when 'year' TrafficControllerSite.make_status now.year.to_s.rjust(4, "0") when 'month' TrafficControllerSite.make_status now.month.to_s.rjust(2, "0") when 'day' TrafficControllerSite.make_status now.day.to_s.rjust(2, "0") when 'hour' TrafficControllerSite.make_status now.hour.to_s.rjust(2, "0") when 'minute' TrafficControllerSite.make_status now.min.to_s.rjust(2, "0") when 'second' TrafficControllerSite.make_status now.sec.to_s.rjust(2, "0") end end |
#handle_s0097(status_code, status_name = nil) ⇒ Object
675 676 677 678 679 680 681 682 683 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 675 def handle_s0097 status_code, status_name=nil case status_name when 'checksum' TrafficControllerSite.make_status '1' when 'timestamp' now = @node.clock.to_s TrafficControllerSite.make_status now end end |
#handle_s0205(status_code, status_name = nil) ⇒ Object
685 686 687 688 689 690 691 692 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 685 def handle_s0205 status_code, status_name=nil case status_name when 'start' TrafficControllerSite.make_status clock.to_s when 'vehicles' TrafficControllerSite.make_status 0 end end |
#handle_s0206(status_code, status_name = nil) ⇒ Object
694 695 696 697 698 699 700 701 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 694 def handle_s0206 status_code, status_name=nil case status_name when 'start' TrafficControllerSite.make_status clock.to_s when 'speed' TrafficControllerSite.make_status 0 end end |
#handle_s0207(status_code, status_name = nil) ⇒ Object
703 704 705 706 707 708 709 710 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 703 def handle_s0207 status_code, status_name=nil case status_name when 'start' TrafficControllerSite.make_status clock.to_s when 'occupancy' TrafficControllerSite.make_status 0 end end |
#handle_s0208(status_code, status_name = nil) ⇒ Object
712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 730 731 732 733 734 735 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 712 def handle_s0208 status_code, status_name=nil case status_name when 'start' TrafficControllerSite.make_status clock.to_s when 'P' TrafficControllerSite.make_status 0 when 'PS' TrafficControllerSite.make_status 0 when 'L' TrafficControllerSite.make_status 0 when 'LS' TrafficControllerSite.make_status 0 when 'B' TrafficControllerSite.make_status 0 when 'SP' TrafficControllerSite.make_status 0 when 'MC' TrafficControllerSite.make_status 0 when 'C' TrafficControllerSite.make_status 0 when 'F' TrafficControllerSite.make_status 0 end end |
#initiate_startup_sequence ⇒ Object
106 107 108 109 110 111 112 113 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 106 def initiate_startup_sequence log "Initiating startup sequence", level: :info reset_modes @dark_mode = false @startup_sequence_active = true @startup_sequence_initiated_at = nil @startup_sequence_pos = nil end |
#move_cycle_counter ⇒ Object
95 96 97 98 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 95 def move_cycle_counter counter = Time.now.to_i % @cycle_time @cycle_counter = counter end |
#move_startup_sequence ⇒ Object
123 124 125 126 127 128 129 130 131 132 133 134 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 123 def move_startup_sequence was = @startup_sequence_pos if @startup_sequence_initiated_at == nil @startup_sequence_initiated_at = Time.now.to_i + 1 @startup_sequence_pos = 0 else @startup_sequence_pos = Time.now.to_i - @startup_sequence_initiated_at end if @startup_sequence_pos >= @startup_sequence.size end_startup_sequence end end |
#output_states ⇒ Object
136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 136 def output_states return unless @live_output str = @signal_groups.map do |group| state = group.state s = "#{group.c_id}:#{state}" if state =~ /^[1-9]$/ s.colorize(:green) elsif state =~ /^[NOP]$/ s.colorize(:yellow) elsif state =~ /^[ae]$/ s.colorize(:light_black) elsif state =~ /^[f]$/ s.colorize(:yellow) elsif state =~ /^[g]$/ s.colorize(:red) else s.colorize(:red) end end.join ' ' modes = '.'*9 modes[0] = 'B' if @booting modes[1] = 'S' if @startup_sequence_active modes[2] = 'D' if @dark_mode modes[3] = 'Y' if @yellow_flash modes[4] = 'M' if @manual_control modes[5] = 'F' if @fixed_time_control modes[6] = 'R' if @all_red modes[7] = 'I' if @isolated_control modes[8] = 'P' if @police_key != 0 plan = "P#{@plan}" File.open @live_output, 'w' do |file| file.puts "#{modes} #{plan.rjust(2)} #{@cycle_counter.to_s.rjust(3)} #{str}\r" end end |
#reset ⇒ Object
40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 40 def reset reset_modes @cycle_counter = 0 @plan = 1 @intersection = 0 @emergency_route = false @emergency_route_number = 0 @traffic_situation = 0 @inputs = '0'*@num_inputs @input_activations = '0'*@num_inputs @input_results = '0'*@num_inputs @day_time_table = {} @startup_sequence_active = false @startup_sequence_initiated_at = nil @startup_sequence_pos = 0 @time_int = nil end |
#reset_modes ⇒ Object
26 27 28 29 30 31 32 33 34 35 36 37 38 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 26 def reset_modes @dark_mode = true @yellow_flash = false @booting = false @is_starting = false @control_mode = 'control' @manual_control = false @fixed_time_control = false @isolated_control = false @yellow_flash = false @all_red = false @police_key = 0 end |
#set_fixed_time_control(status) ⇒ Object
342 343 344 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 342 def set_fixed_time_control status @fixed_time_control = status end |
#set_input(i, value) ⇒ Object
337 338 339 340 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 337 def set_input i, value return unless i>=0 && i<@num_inputs @inputs[i] = (arg['value'] ? '1' : '0') end |
#startup_state ⇒ Object
100 101 102 103 104 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 100 def startup_state return unless @startup_sequence_active return unless @startup_sequence_pos @startup_sequence[ @startup_sequence_pos ] end |
#switch_mode(mode) ⇒ Object
357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 357 def switch_mode mode log "Switching to mode #{mode}", level: :info case mode when 'NormalControl' initiate_startup_sequence if @yellow_flash || @dark_mode @yellow_flash = false @dark_mode = false when 'YellowFlash' @yellow_flash = true @dark_mode = false when 'Dark' @yellow_flash = false @dark_mode = true end mode end |
#switch_plan(plan) ⇒ Object
346 347 348 349 350 351 352 353 354 355 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 346 def switch_plan plan plan_nr = plan.to_i if plan_nr == 0 log "Switching to plan selection by time table", level: :info else plan = find_plan plan_nr log "Switching to plan #{plan_nr}", level: :info end @plan = plan_nr end |
#timer(now) ⇒ Object
82 83 84 85 86 87 88 89 90 91 92 93 |
# File 'lib/rsmp/tlc/traffic_controller.rb', line 82 def timer now # TODO use monotone timer, to avoid jumps in case the user sets the system time time = Time.now.to_i return if time == @time_int @time_int = time move_cycle_counter move_startup_sequence if @startup_sequence_active @signal_groups.each { |group| group.timer } output_states end |